import types
import sympycore

def objsize(obj):
    """ Recursively compute memory size of the object in bytes.

    Returns
    -------
    size : int
      Number of bytes that the object consumes in memory.
    """
    import numpy
    if obj is None:
        return 0 # because None is singleton
    if isinstance (obj, numpy.ndarray):
        return 100+obj.nbytes
    if isinstance (obj, (int, float, bool)): 
        return 24
    if isinstance (obj, (long,complex)): 
        return 32
    if isinstance (obj, tuple): 
        sz = 56
        sz += len (obj)*8
        sz += sum (map (objsize, obj))
        return sz
    if isinstance (obj, list): 
        sz = 136 if obj else 72
        sz += sum (map (objsize, obj))
        return sz
    if isinstance (obj, str):
        sz = 40
        sz += (len(obj) // 8)*8 if obj else 0
        return sz
    if isinstance (obj, dict):
        sz = 280
        for k,v in obj.iteritems ():
            sz += objsize(k) + objsize(v)
        return sz
    if isinstance (obj, set):
        sz = 232
        sz += sum (map (objsize, obj))
        return sz
    if isinstance(obj, types.InstanceType):
        sz = 72
        for k,v in obj.__dict__.iteritems ():
            sz += objsize(k) + objsize(v)
        return sz
    if isinstance(obj, object):
        sz = 64
        for k,v in obj.__dict__.iteritems ():
            sz += objsize(k) + objsize(v)
        return sz
    if obj is None:
        return 16
    raise NotImplementedError ('objsize for %s' % (type (obj)))

def obj2num(s, abs_tol=1e-16):
    if isinstance(s, str):
        f = eval(s)
    else:
        f = s
    #return float(f) # will induce numerical errors and incorrect rank for GJE algorithm.
    i = int (f)
    if i==f:
        return i
    return sympycore.f2q(f, abs_tol)

def time2str(s, last_unit='s'):
    """ Return human readable time string from seconds.

    Examples
    --------
    >>> from iocbio.utils import time_to_str
    >>> print time_to_str(123000000)
    3Y10M24d10h40m
    >>> print time_to_str(1230000)
    14d5h40m
    >>> print time_to_str(1230)
    20m30.0s
    >>> print time_to_str(0.123)
    123ms
    >>> print time_to_str(0.000123)
    123us
    >>> print time_to_str(0.000000123)
    123ns

    """
    seconds_in_year = 31556925.9747 # a standard SI year
    orig_s = s
    years = int(s / (seconds_in_year))
    r = []
    if years:
        r.append ('%sY' % (years))
        s -= years * (seconds_in_year)
    if last_unit=='Y': s = 0
    months = int(s / (seconds_in_year/12.0))
    if months:
        r.append ('%sM' % (months))
        s -= months * (seconds_in_year/12.0)
    if last_unit=='M': s = 0
    days = int(s / (60*60*24))
    if days:
        r.append ('%sd' % (days))
        s -= days * 60*60*24
    if last_unit=='d': s = 0
    hours = int(s / (60*60))
    if hours:
        r.append ('%sh' % (hours))
        s -= hours * 60*60
    if last_unit=='h': s = 0
    minutes = int(s / 60)
    if minutes:
        r.append ('%sm' % (minutes))
        s -= minutes * 60
    if last_unit=='m': s = 0
    seconds = int(s)
    if seconds:
        r.append ('%ss' % (seconds))
        s -= seconds
    if last_unit=='s': s = 0
    mseconds = int(s*1000)
    if mseconds:    
        r.append ('%sms' % (mseconds))
        s -= mseconds / 1000    
    if last_unit=='ms': s = 0
    useconds = int(s*1000000)
    if useconds:
        r.append ('%sus' % (useconds))
        s -= useconds / 1000000
    if last_unit=='us': s = 0
    nseconds = int(s*1000000000)
    if nseconds:
        r.append ('%sns' % (nseconds))
        s -= nseconds / 1000000000
    if not r:
        return '0'
    return ''.join(r)



'''
.. module:: updaters
   :platform: Linux
   :synopsis: Wrapper package module for a collection of tools required for doing intelligent and dynamic DSGE model instance updating
              at runtime. The version in this module is for the one-off updater behaviour.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


''' 



# Import the pymaclab module into its namespace, also import os module
import pymaclab as pm
from pymaclab.modfiles import models
import numpy as np
from matplotlib import pyplot as plt
from matplotlib import rc
rc('text', usetex=True)

def test2():
    # Instantiate a new DSGE model instance like so
    rbc1 = pm.newMOD(models.stable.rbc1_res)

    # Create an array representing a finely-spaced range of possible impatience values
    # Then convert to corresponding steady state gross real interest rate value
    betarr = np.arange(0.8,0.99,0.001)
    betarr = 1.0/betarr


    ss_capital = []
    for betar in betarr:
	rbc1.updaters.paramdic['R_bar'] = betar
	rbc1.sssolvers.fsolve.solve()
	ss_capital.append(rbc1.sssolvers.fsolve.fsout['k_bar'])

    # Create a nice figure
    fig1 = plt.figure()
    plt.grid()
    plt.title('Plot of steady state physical capital against R\_bar')
    plt.xlabel(r'Steady state gross real interest rate')
    plt.ylabel(r'Steady State of physical capital')
    plt.plot(betarr,ss_capital,'k-')
    #plt.show()



from sympycore.arithmetic.evalf import *
from sympycore.arithmetic.evalf import mpmath, compile_mpmath
from sympycore.calculus import Symbol, I, Number, Exp, Sin, Cos, E, pi
import math
import cmath

def test_evalf():
    expr1 = Number(1)/3
    expr2 = Sin(E)**2 + Cos(E)**2 - 1
    expr3 = Exp(I) - Cos(1) - I*Sin(1)
    assert abs(pi.evalf(15) - math.pi) < 1e-14, str( abs(pi.evalf(15) - math.pi))
    assert abs(expr1.evalf(30) - expr1) < 1e-29
    assert abs(expr2.evalf(30)) < 1e-29, `abs(expr2.evalf(30))`
    assert abs(expr2.evalf(100)) < 1e-99
    assert abs(expr2.evalf(300)) < 1e-99
    #assert abs(expr3.evalf(20)) < 1e-19

def test_compiled():
    x = Symbol('x')
    y = Symbol('y')
    f1 = compile_mpmath([], Exp(2))
    f2 = compile_mpmath('x', Exp(x))
    f3 = compile_mpmath(['x', 'y'], Cos(x)+Sin(y)*I)
    mpmath.mp.dps = 15
    assert abs(f1() - math.exp(2)) < 1e-14
    assert abs(f2(2) - math.exp(2)) < 1e-14
    assert abs(f3(3,4) - (cmath.cos(3)+cmath.sin(4)*1j)) < 1e-14



import pymaclab as pm
from pymaclab.modfiles import models

def test3():
    print "Now Loading and differentiating model rbc1..."
    rbc1 = pm.newMOD(models.stable.rbc1_res,mesg=True,ncpus='auto')
    print
    print "Now loading and differentiating model rbc2..."
    rbc2 = pm.newMOD(models.stable.rbc2,mesg=True,ncpus='auto')
    print
    print "Now loading and differentiating model merz..."
    merz = pm.newMOD(models.stable.merz,mesg=True,ncpus='auto')
    print
    print "Now loading and differentiating model cee..."
    cee  = pm.newMOD(models.testing.cee,mesg=True,ncpus='auto')



# File setup.py
def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    from numpy.distutils.system_info import get_info
    config = Configuration('linalg',parent_package,top_path)
    return config

if __name__ == "__main__":
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



'''
.. module:: _modparser
   :platform: Linux
   :synopsis: This is the (private) module responsible for collecting raw information from the DSGE model templated files. In here
              we don't make much use of Regex patterns as we are only extracting raw lines from the mod files.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''
import os
import re

modfpath = "" #TODO: get rid of these globals, make a config file

#TODO: make sure that everything gets evaluated in float terms
#for instance, in old cee.txt 1.03**(1/4) does int division
class ParsedMod(object):
    """
    Bunch pattern ParsedMod object

    Attributes
    ----------
    filename : str
        .mod file path and name
    secs : dict
        The sections of the .mod file and their contents
    filestring : str
        The .mod file as a string
    lines : list
        filestring split by lines
    numlines
        Length of line
    """
    def __init__(self, **kwds):
        self.__dict__.update(kwds)

def locate(stringlines,varlist):
    '''
    A function, takes stringlines (split file) and a varlist
    and then creates a location dictionary for location of strings in file
    '''
    locdic = {}
    for x in varlist:
        row_iter=0
        while row_iter < len(stringlines):
            # not case sensitive
            if x[0].lower() in stringlines[row_iter].lower():
                locdic[x[1]] = row_iter
                break
            else:
                row_iter += 1
    return locdic


def read_nativefmt(filestring, fname):
    """
    Reads native .mod format and return as ParsedMod class.
    """
    secnames = [['%Model Information','info','INF_loc'],
                ['%Parameters','params','PA_loc'],
                ['%Variable Vectors','varvec','VV_loc'],
                ['%Boundary Conditions','bocond','BC_loc'],
                ['%Variable Substitution Non-Linear System','vsfocs','VSFO_loc'],
                ['%Non-Linear First-Order Conditions','focs','FO_loc'],
                ['%Steady States [Closed Form]','closedformss','SS_loc'],
                ['%Steady State Non-Linear System [Manual]','manualss','SSM_loc'],
                ['%Log-Linearized Model Equations','modeq','ME_loc'],
                ['%Variance-Covariance Matrix','vcvm','VCM_loc']]

    lines = filestring.splitlines()
    numlines = len(lines)
    secs = {}
    locdic = locate(lines,secnames)

    for x in secnames:
        section_name = x[1]
        list_tmp1 = []
        list_tmp2 = []
        row_iter = locdic[section_name]+1 # section starts 1 line after header
        for line in lines[row_iter:]:
            line = re.sub("\s+", "", line) # strip out any whitespace char
            if not line.startswith(('#','%')) and line != "":
                list_tmp1.append(lines[row_iter])
                list_tmp2.append(lines[row_iter])
                # really hold these duplicates if not labels?
            elif line.startswith('#'): # grab labels
                list_tmp2.append(lines[row_iter])
            elif line.startswith('%'):
                break
            row_iter += 1
        secs[section_name] = (list_tmp1,list_tmp2)
        
    #TODO: what do we actually _need_ and do we need a class?
    return ParsedMod(filename=fname, secs=secs, filestring=filestring, 
            lines=lines, numlines=len(lines))

            

def parse_mod(fname, modfmt='pymaclab'):
#class MODparser(object):
    """
    Parses a .mod file.

    Parameters
    ----------
    fname : str
        Filename to parse. Could also be the model info as string itself.
    modfmt : str {'pymaclab'}
        Only native .mod formats are currently supported.

    Returns
    -------
    ParsedMod class.
    """
    if os.path.exists(fname):
        with open(os.path.join(modfpath, fname), 'r') as txtfile:
            filestring = txtfile.read()
            if modfmt == 'pymaclab':
                parsed_mod = read_nativefmt(filestring, fname)
            else:
                raise ValueError("modfmt %s not understood" % modfmt)
    # Assume here that the model information was directly passed as a string
    elif '%Model Description+++++++' in fname:
        if modfmt == 'pymaclab':
            parsed_mod = read_nativefmt(fname, fname)
        else:
            raise ValueError("modfmt %s not understood" % modfmt)
    #  Otherwise don't know what to do with the input and throw error
    else:
        raise ValueError("modfmt %s not understood" % modfmt)
        
    return parsed_mod



# File setup.py
def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('filters',parent_package,top_path)

    config.add_extension('_hpfilter',
                         sources = ['src/hpfilter.f'])
    config.add_extension('_bkfilter',
                         sources = ['src/bkfilter.f90'])
    return config

if __name__ == "__main__":
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



"""SympyCore - An efficient pure Python Computer Algebra System.

See http://sympycore.googlecode.com/ for more information.
"""
__docformat__ = "restructuredtext"
__date__ = '2008'
__author__ = 'Pearu Peterson, Fredrik Johansson'
__license__ = 'New BSD License'

from .version import version as __version__

from .core import classes, defined_functions, DefinedFunction, Expr, Pair, IntegerList
from .core import init_module

import heads

init_module.execute()

from arithmetic import *

from ring import *

from basealgebra import *
from algebras import *
from logic import *
from sets import *
from calculus import *
from polynomials import *
from matrices import *
from physics import *
from functions import *

CollectingField = CommutativeRing

### Initialize sympycore subpackage namespaces
init_module.execute()

def profile_expr(expr):
    """ Printout the profiler information for executing ``expr``.
    """
    import sys
    import hotshot, hotshot.stats
    prof = hotshot.Profile("/tmp/sympycore_stones.prof")
    frame = sys._getframe(1)
    g = frame.f_globals
    l = frame.f_locals
    def foo():
        exec expr in g,l
    prof.runcall(foo)
    prof.close()
    stats = hotshot.stats.load("/tmp/sympycore_stones.prof")
    #stats.strip_dirs()
    stats.sort_stats('time','calls','time')
    stats.print_stats(40)
    return stats

# This function is taken from sympy (and originally sage?)
def var(s):
    """
    var('x y z') creates symbols x, y, z
    """
    import inspect
    frame = inspect.currentframe().f_back
    try:
        if not isinstance(s, list):
            s = s.split(" ")
        res = []
        for t in s:
            # skip empty strings
            if not t:
                continue
            sym = Symbol(t)
            frame.f_globals[t] = sym
            res.append(sym)
        res = tuple(res)
        if len(res) == 0:   # var('')
            res = None
        elif len(res) == 1: # var('x')
            res = res[0]
                            # otherwise var('a b ...')
        return res
    finally:
        # we should explicitly break cyclic dependencies as stated in inspect
        # doc
        del frame

FD = FDFactory(CalculusDifferentialRing)
D = DFactory(CalculusDifferentialRing)

sin = CalculusFunctionRing(heads.CALLABLE, Sin)
arcsin = CalculusFunctionRing(heads.CALLABLE, ArcSin)
cos = CalculusFunctionRing(heads.CALLABLE, Cos)
tan = CalculusFunctionRing(heads.CALLABLE, Tan)
cot = CalculusFunctionRing(heads.CALLABLE, Cot)
ln = CalculusFunctionRing(heads.CALLABLE, Ln)
log = CalculusFunctionRing(heads.CALLABLE, Log)
exp = CalculusFunctionRing(heads.CALLABLE, Exp)

class _Tester:

    def __init__(self):
        self.check_testing()
        
    def test(self, nose_args=''):
        """ Run sympycore tests using nose.
        """
        import os
        import sys
        import nose
        #mpmath = sys.modules['sympycore.arithmetic.mpmath']
        d = os.path.dirname(os.path.abspath(__file__))
        cmd = '%s %s %s %s' % (sys.executable, nose.core.__file__, d, nose_args)
        print >>sys.stderr, 'Running %r' % cmd
        s = os.system(cmd)
        if s:
            print >>sys.stderr, "TESTS FAILED"

    def check_testing(self):
        import os, sys
        try:
            import nose
        except ImportError:
            return
        if sys.platform=='win32':
            m = lambda s: s.lower()
        else:
            m = lambda s: s
        argv2 = [nose.core.__file__, os.path.dirname(os.path.abspath(__file__))]
        if map(m, sys.argv[:2]) == map(m, argv2):
            self.show_config()

    def show_config(self):
        import os, sys

        print >>sys.stderr, 'Python version: %s' % (sys.version.replace('\n',''))

        nose = None
        minimum_nose_version = (0,10,0)
        try:
            import nose
            if nose.__versioninfo__ < minimum_nose_version:
                nose = None
        except:
            pass
        if nose is None:
            msg = 'Need nose >= %d.%d.%d for tests - see ' \
              'http://somethingaboutorange.com/mrl/projects/nose' % \
              minimum_nose_version
            raise ImportError(msg)            
        else:
            print >>sys.stderr, 'nose version: %d.%d.%d' % nose.__versioninfo__
            print >>sys.stderr, 'nose is installed in %s' % (os.path.dirname(nose.__file__))

        if 'sympycore.expr_ext' in sys.modules:
            s = 'compiled'
        else:
            s = 'pure'
        print >>sys.stderr, 'sympycore version: %s (%s)' % (__version__, s)
        print >>sys.stderr, 'sympycore is installed in %s' % (os.path.dirname(__file__))

        mpmath = sys.modules['sympycore.arithmetic.mpmath']
        fn = os.path.join(os.path.dirname(mpmath.__file__), 'REVISION')
        l = []
        backend = mpmath.libmp.backend.BACKEND
        l.append ('backend=%s' % (backend))
        if os.path.isfile(fn):
            f = open(fn, 'r')
            l.append('revision=%s' % (f.read().strip()))
            f.close()
        else:
            rev = ''
        print >>sys.stderr, 'mpmath version: %s (%s)' % (mpmath.__version__, ', '.join(l))
        print >>sys.stderr, 'mpmath is installed in %s' % (os.path.dirname(mpmath.__file__))

        
        if backend=='gmpy':
            gmpy = mpmath.libmp.backend.gmpy
            print >>sys.stderr, 'gmpy version: %s' % (gmpy.version())
            print >>sys.stderr, 'gmpy is installed in %s' % (os.path.dirname(gmpy.__file__))


test = _Tester().test
del _Tester




""" Provides BasicAlgebra class.
"""

__docformat__ = "restructuredtext"
__all__ = ['Algebra', 'SymbolicEquality']

from ..core import classes, Expr, defined_functions
from ..utils import LT, GT, LE, GE, NE, EQ, SYMBOL, NUMBER
from ..heads import CALLABLE, APPLY

symbolic_comparison_map = dict(
    equality = dict(__eq__=EQ, __ne__=NE),
    inequality = dict(__lt__=LT,__le__=LE, __gt__=GT,__ge__=GE)
    )

class SymbolicEquality:
    """ Contex for logical operations.

    In the ``SymbolicEquality(<Algebra class>)`` context relational
    operations return ``Logic`` instances instead of computing
    lexicographic value of relational operations.

    For example,

    >>> x = Calculus('x')
    >>> with SymbolicEquality(Calculus):
    >>>     print x==x
    ...     
    ...     
    x==x
    >>> print x==x
    True

    Add ``from __future__ import with_statement`` to the header of
    python file when using Python version 2.5.
    
    """

    def __init__(self, *classes):
        self.classes = classes

    def __enter__(self):
        for cls in self.classes:
            cls.enable_symbolic_comparison()

    def __exit__(self, type, value, tb):
        for cls in self.classes:
            cls.disable_symbolic_comparison()
        return tb is None


class Algebra(Expr):
    """ Represents an element of an algebraic structure.

    This class collects implementation specific methods of algebra
    classes.
    
    For implemented algebras, see:

      Verbatim
      CollectingField
    
    New algebras may need to redefine the following methods::
    
      __new__(cls, ...)
      convert(cls, obj, typeerror=True)
      convert_coefficient(cls, obj, typeerror=True)
      convert_exponent(cls, obj, typeerror=True)
      convert_operand(self, obj, typeerror=True)
      as_verbatim(self)
      as_algebra(self, cls, typeerror=True)

    and the following properties::

      args(self)
      func(self)

    """
    #__slots__ = ['_str_value']

    _str_value = None

    coefftypes = (int, long)
    exptypes = (int, long)

    algebra_options = dict(evaluate_addition = True)

    def __str__(self):
        h, d = self.pair
        return h.data_to_str_and_precedence(type(self), d)[0]
        s = self._str_value
        if s is None:
            head, data = self.pair
            s = head.data_to_str(type(self), data, 0.0)
            if not self.is_writable:
                self._str_value = s
        return s

    def __repr__(self):
        s = str(self)
        return '%s(%r)' % (type(self).__name__, s)

    def __nonzero__(self):
        return not not self.data

    @classmethod
    def enable_symbolic_comparison(cls, name = 'equality'):
        """ Enable returning Logic instances from relational methods.

        Argument ``name`` can be 'equality' or 'inequality'.
        """
        logic_map = symbolic_comparison_map.get(name)
        if logic_map is None:
            raise ValueError('Unknown symbolic comparison name: %r. Valid names are %s'\
                             % (name, ', '.join([repr(k) for k in symbolic_comparison_map])))
        logic_map_name = '_%s_map' % (name)
        d = getattr(cls, logic_map_name, None)
        if d is None:
            d = {}
            setattr(cls, logic_map_name, d)
            for mthname, head in logic_map.items():
                d[mthname] = getattr(cls, mthname, None)
                setattr(cls, mthname, lambda self, other, head=head: classes.Logic(head, (self, other)))

    @classmethod
    def disable_symbolic_comparison(cls, name = 'equality'):
        """ Disable returning Logic instances from relational methods.

        Argument ``name`` can be 'equality' or 'inequality'.
        """
        logic_map = symbolic_comparison_map.get(name)
        if logic_map is None:
            raise ValueError('Unknown symbolic comparison name: %r. Valid names are %s'\
                             % (name, ', '.join([repr(k) for k in symbolic_comparison_map])))
        logic_map_name = '_%s_map' % (name)
        d = getattr(cls, logic_map_name, None)
        if d is None:
            return
        for mthname, mth in d.items():
            if mth is None:
                delattr(cls, mthname)
            else:
                setattr(cls, mthname, mth)
        delattr(cls, logic_map_name)

    def as_tree(self, tab='', level=0):
        return self.as_verbatim().as_tree(tab,level)

    @classmethod
    def get_operand_algebra(cls, head, index=0):
        """ Return algebra class for index-th operand of operation with head.
        """
        return cls

    @classmethod
    def handle_get_operand_algebra_failure(cls, head, index):
        raise TypeError('Cannot convert %s-th %s operand to %s algebra' % (index, repr(head), cls.__name__))

    @classmethod
    def convert(cls, obj, typeerror=True):
        """ Convert obj to algebra element.

        With typeerror=False NotImplemented is returned when
        conversion is not possible, otherwise a TypeError is raised.
        
        Set typeerror=False when calling from operation methods like __add__,
        __mul__, etc.
        """
        tobj = type(obj)
        # check if obj is already algebra element:
        if tobj is cls:
            return obj
        
        # check if obj belongs to coefficient algebra
        r = cls.convert_number(obj, typeerror=False)
        if r is not NotImplemented:
            return cls.Number(r)

        # parse algebra expression from string:
        if isinstance(obj, (str, unicode, Verbatim)):
            return Verbatim.convert(obj).as_algebra(cls)

        # as a last resort, convert from another algebra:
        if isinstance(obj, Algebra):
            result = obj.as_algebra(cls, typeerror=typeerror)
            if result is not NotImplemented:
                return result
            
        return cls.handle_convert_failure('algebra', obj, typeerror)

    @classmethod
    def convert_exponent(cls, obj, typeerror=True):
        """ Convert obj to exponent algebra.
        """
        if isinstance(obj, cls.exptypes) or isinstance(obj, cls):
            return obj
        return cls.handle_convert_failure('exponent algebra', obj, typeerror, 'expected int|long|'+cls.__name__)

    @classmethod
    def convert_coefficient(cls, obj, typeerror=True):
        """ Convert obj to coefficient algebra.
        """
        # Use convert_number instead.
        if isinstance(obj, cls.coefftypes):
            return obj
        return cls.handle_convert_failure('coefficient algebra', obj, typeerror, 'expected int|long')

    @classmethod
    def handle_convert_failure(cls, name, obj, typeerror, hint=''):
        if typeerror:
            if hint:
                hint = ' (%s)' % (hint)
            errmsg = 'failed to convert %r to %r %s%s'\
                     % (obj.__class__.__name__, cls.__name__, name, hint)
            raise TypeError(errmsg)
        else:
            return NotImplemented

    @classmethod
    def convert_symbol(cls, obj, typeerror=True):
        """ Convert obj to algebra symbol or predefined function or
        constant.
        """
        r = cls.get_predefined_symbols(obj)
        if r is not None:
            return r
        return cls(SYMBOL, obj)

    @classmethod
    def convert_number(cls, obj, typeerror=True):
        """ Convert obj to algebra number.

        On success, the result is assumed to be used in ``Algebra.Number(<result>)``.

        On failure, the TypeError is raised if typeerror is True (default).
        Otherwise, NotImplemented is returned.
        """
        #XXX: return cls(NUMBER, obj)
        return cls.convert_coefficient(obj, typeerror)

    def convert_operand(self, obj, typeerror=True):
        """ Convert obj to operand.

        Used when operand algebra depends on expression head.
        """
        return self.convert(obj, typeerror=typeerror)

    @classmethod
    def convert_function(cls, name, args):
        """ Convert verbatim applied function call to algebra.
        """
        func = getattr(defined_functions, name.title(), None)
        arg_algebras = func.get_argument_algebras()
        return func(*[cls(a) for a,cls in zip(args, arg_algebras)])
    
    def as_verbatim(self):
        def as_verbatim(cls, head, data, target):
            return cls(head, data)
        return self.head.walk(as_verbatim, Verbatim, self.data, self)

    def as_algebra(self, cls, typeerror=True):
        """ Convert algebra to another algebra.

        This method uses default conversation via verbatim algebra that
        might not be the most efficient. For efficiency, algebras should
        redefine this method to implement direct conversation.
        """
        # todo: cache verbatim algebras
        if cls is classes.Verbatim:
            return self.as_verbatim()
        return self.as_verbatim().as_algebra(cls)

    @classmethod
    def get_predefined_symbols(cls, name):
        return getattr(defined_functions, name, None)

    @property
    def func(self):
        """ Returns a callable such that ``self.func(*self.args) == self``.
        """
        raise NotImplementedError('%s must define property func'      #pragma NO COVER
                                  % (self.__class__.__name__))                   #pragma NO COVER

    @property
    def args(self):
        """ Returns a sequence such that ``self.func(*self.args) == self``.
        """
        raise NotImplementedError('%s must define property "args"'      #pragma NO COVER
                                  % (self.__class__.__name__))        #pragma NO COVER

    @classmethod
    def Symbol(cls, obj):
        """ Construct algebra symbol directly from obj.
        """
        return cls(SYMBOL, obj)

    @classmethod
    def Number(cls, num):
        """ Construct algebra number directly from obj.
        """
        return cls(NUMBER, num)

    @classmethod
    def Apply(cls, func, *args, **kwargs):
        convert= cls.convert
        fcls = cls.get_function_algebra()
        if isinstance(func, Expr):
            pass
        elif callable(func):
            func = fcls(CALLABLE, func)
        else:
            func = fcls(SYMBOL, func)
        new_args = map(convert, args)
        for k, v in kwargs.items():
            new_args.append(cls(KWARG, (convert(k), convert(v))))
        return cls(APPLY, (func, tuple(new_args)))

    _cache_symbols = None

    @property
    def symbols(self):
        """ Return a set of atomic subexpressions in a symbolic object.
        """
        symbols = self._cache_symbols
        if symbols is None:
            def scan_for_symbols(cls, head, data, target):
                if head is SYMBOL:
                    target.add(cls(head, data)) # introduce expr argument to scan
            head, data = self.pair
            symbols = set([])
            head.scan(scan_for_symbols, type(self), data, symbols)
            self._cache_symbols = symbols
            #todo: make self read-only
        return symbols

    _cache_symbols_data = None
    @property
    def symbols_data(self):
        symbols = self._cache_symbols_data
        if symbols is None:
            symbols = set([s.data for s in self.symbols])
            self._cache_symbols_data = symbols
        return symbols

    def has_symbol(self, symbol):
        symbol = type(self)(symbol)
        return symbol in self.symbols

    def has(self, obj):
        """ Return True if self contains atomic expression obj.
        """
        convert = self.convert
        return convert(obj) in self.symbols

    def match(self, pattern, *wildcards, **options):
        """
        Pattern matching.

        Return None when expression (self) does not match with
        pattern. Otherwise return a dictionary such that

        ::

          pattern.subs_dict(self.match(pattern, *wildcards)) == self

        Don't redefine this method, redefine ``matches(..)`` method instead.

        :See:
          http://wiki.sympy.org/wiki/Generic_interface#Pattern_matching
        """
        exclude = options.get('exclude',[])
        pattern = self.convert(pattern)
        wild_expressions = []
        wild_predicates = []
        for w in wildcards:
            if type(w) in [list, tuple]:
                assert len(w)==2,`w`
                s, func = w
            else:
                if exclude:
                    s, func = w, lambda x: x not in exclude
                else:
                    s, func = w, True
            s = self.convert(s)
            wild_expressions.append(s)
            wild_predicates.append(func)
        if wild_expressions:
            wild_info = (wild_expressions, wild_predicates)
        else:
            wild_info = None
        return pattern.matches(self, {}, wild_info)

    def matches(pattern, expr, repl_dict={}, wild_info=None):
        # check if pattern has a match and return it provided that
        # the match matches with expr:
        v = repl_dict.get(pattern)
        if v is not None:
            if v==expr:
                return repl_dict
            return
        # check if pattern matches with expr:
        if pattern==expr:
            if wild_info and expr in wild_info[0]:
                return
            return repl_dict
        if wild_info:
            wild_expressions, wild_predicates = wild_info
            # check if pattern is a wild expression
            if pattern in wild_expressions:
                if expr in wild_expressions:
                    # wilds do not match other wilds
                    return
                if expr.symbols.intersection(pattern.symbols):
                    # wild does not match expressions containing wild expressions
                    return
                # wild pattern matches with expr only if predicate(expr) returns True
                predicate = wild_predicates[wild_expressions.index(pattern)]
                if (isinstance(predicate, bool) and predicate) or predicate(expr):
                    repl_dict = repl_dict.copy()
                    repl_dict[pattern] = expr
                    return repl_dict
            pattern_symbols = pattern.symbols
            for w in wild_expressions:
                if w in pattern_symbols:
                    # _matches implements implementation specific matches,
                    # pattern should contain wild symbols, otherwise
                    # _matches would always return None
                    return pattern._matches(expr, repl_dict, wild_info)
        return

    @classmethod
    def _matches_seq(cls, pattern_seq, expr_seq, repl_dict, wild_info):
        n = len(pattern_seq)
        m = len(expr_seq)
        if n!=m:
            return
        elif n==0:
            return repl_dict
        elif n==1:
            p = pattern_seq[0]
            if isinstance(p, cls):
                return p.matches(expr_seq[0], repl_dict, wild_info)
            if p==expr_seq[0]:
                return repl_dict
            return
        def matches(pattern, expr):
            if isinstance(pattern, cls):
                return pattern.matches(expr, repl_dict, wild_info)
            if pattern==expr:
                return repl_dict
            return
        def subs(expr, l):
            if isinstance(expr, cls):
                return expr.subs(l)
            return expr
        for i in xrange(n):
            p = pattern_seq[i]
            e = expr_seq[i]
            r = matches(p, e)
            if r is not None:
                new_pattern_seq = [subs(pattern_seq[j],r.items()) for j in xrange(n) if j!=i]
                new_expr_seq = [expr_seq[j] for j in xrange(n) if j!=i]
                r1 = cls._matches_seq(new_pattern_seq, new_expr_seq, r, wild_info)
                if r1 is not None:
                    return r1
        return

    def _matches(pattern, expr, repl_dict, wild_info):
        if pattern.func == expr.func:
            return pattern._matches_seq(pattern.args, expr.args, repl_dict, wild_info)
        return

    def subs(self, subexpr, newexpr=None):
        """ Substitute a sub-expression with new expression.
        
        There are three usage forms::
        
          obj.subs(subexpr, newexpr)
          obj.subs([(subexpr1, newexpr1), (subexpr2, newexpr2), ..])
          obj.subs(<dict of subexpr:newexpr>)
        """
        cls = type(self)
        if newexpr is not None:
            def subs(cls, head, data, target, subexpr=subexpr, newexpr=newexpr):
                if target==subexpr:
                    if type(newexpr) is not cls:
                        newexpr = cls(newexpr)
                        return newexpr.head.new(cls, newexpr.data)
                    return newexpr
                return target
        elif type(subexpr) is dict:
            def subs(cls, head, data, target, expr_dict=subexpr):
                r = expr_dict.get(target, target)
                if type(r) is not cls:
                    r = cls(r)
                    return r.head.new(cls, r.data)
                return r
        elif type(subexpr) is list:
            def subs(cls, head, data, target, expr_list=subexpr):
                for s, n in expr_list:
                    if s==target:
                        if type(n) is not cls:
                            n = cls(n)
                            return n.head.new(cls, n.data)
                        return n
                return target
        else:
            raise NotImplementedError(`subexpr, newexpr`)
        head, data = self.pair
        return head.walk(subs, cls, data, self)

        convert = lambda x:x
        if newexpr is None:
            if type(subexpr) is dict:
                subexpr = subexpr.iteritems()
            r = self
            for s,n in subexpr:
                r = r._subs(convert(s), convert(n))
            return r
        return self._subs(convert(subexpr), convert(newexpr))

    def _subs(self, subexpr, newexpr):
        head, data = self.pair
        t = type(data)
        cls = type(self)

        if type(subexpr) is not cls:
            if head is SYMBOL:
                if subexpr==data:
                    return self.convert_operand(newexpr)
                return self
        else:
            h, d = subexpr.pair
            if h is head and data==d:
                return self.convert_operand(newexpr)

        if head is SYMBOL:
            return self
        
        if isinstance(data, (tuple, list)):
            r = []
            for a in data:
                if hasattr(a, '_subs'):
                    r.append(a._subs(subexpr, newexpr))
                else:
                    r.append(a)
            return cls(head, t(r))

        if isinstance(data, dict):
            d = dict.__new__(t)
            for t,c in data.iteritems():
                t1 = t._subs(subexpr, newexpr) if hasattr(t,'_subs') else t
                c1 = c._subs(subexpr, newexpr) if hasattr(c,'_subs') else c
                d[t1] = c1
            return cls(head, d)

        return cls(head, data._subs(subexpr, newexpr) if hasattr(data, '_subs') else data)

from .verbatim import Verbatim



from mako.template import Template


template = """\

var ${[x[0].split('(')[0] for x in vardic['endo']['var']+vardic['con']['var']+vardic['exo']['var']].__str__().replace('[','').replace(']','').replace("'","")};
varexo ${[x[2].split('(')[0] for x in vardic['exo']['var']].__str__().replace('[','').replace(']','').replace("'","")};

parameters ${[x for x in paramdic.keys()].__str__().replace('[','').replace(']','').replace("'","")}, ${[x for x in ssidic.keys()].__str__().replace('[','').replace(']','').replace("'","")};

% for keyo in [x for x in paramdic.keys()]:
${keyo} = ${paramdic[keyo]};
% endfor
% for keyo in [x for x in ssidic.keys()]:
${keyo} = ${ssidic[keyo]};
% endfor


model;
% for item in focs_dynare:
0=${item.replace('E(t)|','').replace('LOG','log').replace('EXP','exp').replace('(t)','').replace('(t+1)','(+1)').replace('(t-1)','(-1)').replace('**','^')};
% endfor
end;

<%
initvli = []
allvar = [x[0].split('(')[0] for x in vardic['endo']['var']+vardic['exo']['var']+vardic['con']['var']]
%>
initval;
% for item in ssili:
% if item[0] in [x[0].split('(')[0]+'_bar' for x in vardic['con']['var']] or item[0] in [x[0].split('(')[0]+'_bar' for x in vardic['endo']['var']] or item[0] in [x[0].split('(')[0]+'_bar' for x in vardic['exo']['var']]:
${item[0].replace('_bar','')} = ${item[1]}; <% initvli.append(item[0].replace('_bar','')) %>
% endif
% endfor
% for keyo in [x for x in paramdic.keys() if '_bar' in x]:
% if keyo in [x[0].split('(')[0]+'_bar' for x in vardic['con']['var']] or keyo in [x[0].split('(')[0]+'_bar' for x in vardic['endo']['var']] or keyo in [x[0].split('(')[0]+'_bar' for x in vardic['exo']['var']]:
${keyo.replace('_bar','')} = ${paramdic[keyo]}; <% initvli.append(item[0].replace('_bar','')) %>
% endif
% endfor
% for varo in [x for x in [y for y in allvar if y not in initvli]]:
${varo} = 1.0;
% endfor
end;


order = 1;

% for i1,row in enumerate(range(sigma.shape[0])):
% if i1 == 0 and sigma.shape[0] == 1:
vcov = [${sigma[i1].__str__().replace('[','').replace(']','').replace("'","")} ];
% elif i1 == 0 and sigma.shape[0] != 1:
vcov = [${sigma[i1].__str__().replace('[','').replace(']','').replace("'","")};
% elif i1 != 0 and i1 != sigma.shape[0]-1:
${sigma[i1].__str__().replace('[','').replace(']','').replace("'","")};
% elif i1 != 0 and i1 == sigma.shape[0]-1:
${sigma[i1].__str__().replace('[','').replace(']','').replace("'","")} ];
% endif
% endfor


"""

mako_dynare_template = Template(template)


from ..libmp.backend import xrange

class SpecialFunctions(object):
    """
    This class implements special functions using high-level code.

    Elementary and some other functions (e.g. gamma function, basecase
    hypergeometric series) are assumed to be predefined by the context as
    "builtins" or "low-level" functions.
    """
    defined_functions = {}

    # The series for the Jacobi theta functions converge for |q| < 1;
    # in the current implementation they throw a ValueError for
    # abs(q) > THETA_Q_LIM
    THETA_Q_LIM = 1 - 10**-7

    def __init__(self):
        cls = self.__class__
        for name in cls.defined_functions:
            f, wrap = cls.defined_functions[name]
            cls._wrap_specfun(name, f, wrap)

        self.mpq_1 = self._mpq((1,1))
        self.mpq_0 = self._mpq((0,1))
        self.mpq_1_2 = self._mpq((1,2))
        self.mpq_3_2 = self._mpq((3,2))
        self.mpq_1_4 = self._mpq((1,4))
        self.mpq_1_16 = self._mpq((1,16))
        self.mpq_3_16 = self._mpq((3,16))
        self.mpq_5_2 = self._mpq((5,2))
        self.mpq_3_4 = self._mpq((3,4))
        self.mpq_7_4 = self._mpq((7,4))
        self.mpq_5_4 = self._mpq((5,4))
        self.mpq_1_3 = self._mpq((1,3))
        self.mpq_2_3 = self._mpq((2,3))
        self.mpq_4_3 = self._mpq((4,3))
        self.mpq_1_6 = self._mpq((1,6))
        self.mpq_5_6 = self._mpq((5,6))
        self.mpq_5_3 = self._mpq((5,3))

        self._misc_const_cache = {}

        self._aliases.update({
            'phase' : 'arg',
            'conjugate' : 'conj',
            'nthroot' : 'root',
            'polygamma' : 'psi',
            'hurwitz' : 'zeta',
            #'digamma' : 'psi0',
            #'trigamma' : 'psi1',
            #'tetragamma' : 'psi2',
            #'pentagamma' : 'psi3',
            'fibonacci' : 'fib',
            'factorial' : 'fac',
        })

        self.zetazero_memoized = self.memoize(self.zetazero)

    # Default -- do nothing
    @classmethod
    def _wrap_specfun(cls, name, f, wrap):
        setattr(cls, name, f)

    # Optional fast versions of common functions in common cases.
    # If not overridden, default (generic hypergeometric series)
    # implementations will be used
    def _besselj(ctx, n, z): raise NotImplementedError
    def _erf(ctx, z): raise NotImplementedError
    def _erfc(ctx, z): raise NotImplementedError
    def _gamma_upper_int(ctx, z, a): raise NotImplementedError
    def _expint_int(ctx, n, z): raise NotImplementedError
    def _zeta(ctx, s): raise NotImplementedError
    def _zetasum_fast(ctx, s, a, n, derivatives, reflect): raise NotImplementedError
    def _ei(ctx, z): raise NotImplementedError
    def _e1(ctx, z): raise NotImplementedError
    def _ci(ctx, z): raise NotImplementedError
    def _si(ctx, z): raise NotImplementedError
    def _altzeta(ctx, s): raise NotImplementedError

def defun_wrapped(f):
    SpecialFunctions.defined_functions[f.__name__] = f, True

def defun(f):
    SpecialFunctions.defined_functions[f.__name__] = f, False

def defun_static(f):
    setattr(SpecialFunctions, f.__name__, f)

@defun_wrapped
def cot(ctx, z): return ctx.one / ctx.tan(z)

@defun_wrapped
def sec(ctx, z): return ctx.one / ctx.cos(z)

@defun_wrapped
def csc(ctx, z): return ctx.one / ctx.sin(z)

@defun_wrapped
def coth(ctx, z): return ctx.one / ctx.tanh(z)

@defun_wrapped
def sech(ctx, z): return ctx.one / ctx.cosh(z)

@defun_wrapped
def csch(ctx, z): return ctx.one / ctx.sinh(z)

@defun_wrapped
def acot(ctx, z): return ctx.atan(ctx.one / z)

@defun_wrapped
def asec(ctx, z): return ctx.acos(ctx.one / z)

@defun_wrapped
def acsc(ctx, z): return ctx.asin(ctx.one / z)

@defun_wrapped
def acoth(ctx, z): return ctx.atanh(ctx.one / z)

@defun_wrapped
def asech(ctx, z): return ctx.acosh(ctx.one / z)

@defun_wrapped
def acsch(ctx, z): return ctx.asinh(ctx.one / z)

@defun
def sign(ctx, x):
    x = ctx.convert(x)
    if not x or ctx.isnan(x):
        return x
    if ctx._is_real_type(x):
        if x > 0:
            return ctx.one
        else:
            return -ctx.one
    return x / abs(x)

@defun
def agm(ctx, a, b=1):
    if b == 1:
        return ctx.agm1(a)
    a = ctx.convert(a)
    b = ctx.convert(b)
    return ctx._agm(a, b)

@defun_wrapped
def sinc(ctx, x):
    if ctx.isinf(x):
        return 1/x
    if not x:
        return x+1
    return ctx.sin(x)/x

@defun_wrapped
def sincpi(ctx, x):
    if ctx.isinf(x):
        return 1/x
    if not x:
        return x+1
    return ctx.sinpi(x)/(ctx.pi*x)

# TODO: tests; improve implementation
@defun_wrapped
def expm1(ctx, x):
    if not x:
        return ctx.zero
    # exp(x) - 1 ~ x
    if ctx.mag(x) < -ctx.prec:
        return x + 0.5*x**2
    # TODO: accurately eval the smaller of the real/imag parts
    return ctx.sum_accurately(lambda: iter([ctx.exp(x),-1]),1)

@defun_wrapped
def powm1(ctx, x, y):
    mag = ctx.mag
    one = ctx.one
    w = x**y - one
    M = mag(w)
    # Only moderate cancellation
    if M > -8:
        return w
    # Check for the only possible exact cases
    if not w:
        if (not y) or (x in (1, -1, 1j, -1j) and ctx.isint(y)):
            return w
    x1 = x - one
    magy = mag(y)
    lnx = ctx.ln(x)
    # Small y: x^y - 1 ~ log(x)*y + O(log(x)^2 * y^2)
    if magy + mag(lnx) < -ctx.prec:
        return lnx*y + (lnx*y)**2/2
    # TODO: accurately eval the smaller of the real/imag part
    return ctx.sum_accurately(lambda: iter([x**y, -1]), 1)

@defun
def _rootof1(ctx, k, n):
    k = int(k)
    n = int(n)
    k %= n
    if not k:
        return ctx.one
    elif 2*k == n:
        return -ctx.one
    elif 4*k == n:
        return ctx.j
    elif 4*k == 3*n:
        return -ctx.j
    return ctx.expjpi(2*ctx.mpf(k)/n)

@defun
def root(ctx, x, n, k=0):
    n = int(n)
    x = ctx.convert(x)
    if k:
        # Special case: there is an exact real root
        if (n & 1 and 2*k == n-1) and (not ctx.im(x)) and (ctx.re(x) < 0):
            return -ctx.root(-x, n)
        # Multiply by root of unity
        prec = ctx.prec
        try:
            ctx.prec += 10
            v = ctx.root(x, n, 0) * ctx._rootof1(k, n)
        finally:
            ctx.prec = prec
        return +v
    return ctx._nthroot(x, n)

@defun
def unitroots(ctx, n, primitive=False):
    gcd = ctx._gcd
    prec = ctx.prec
    try:
        ctx.prec += 10
        if primitive:
            v = [ctx._rootof1(k,n) for k in range(n) if gcd(k,n) == 1]
        else:
            # TODO: this can be done *much* faster
            v = [ctx._rootof1(k,n) for k in range(n)]
    finally:
        ctx.prec = prec
    return [+x for x in v]

@defun
def arg(ctx, x):
    x = ctx.convert(x)
    re = ctx._re(x)
    im = ctx._im(x)
    return ctx.atan2(im, re)

@defun
def fabs(ctx, x):
    return abs(ctx.convert(x))

@defun
def re(ctx, x):
    x = ctx.convert(x)
    if hasattr(x, "real"):    # py2.5 doesn't have .real/.imag for all numbers
        return x.real
    return x

@defun
def im(ctx, x):
    x = ctx.convert(x)
    if hasattr(x, "imag"):    # py2.5 doesn't have .real/.imag for all numbers
        return x.imag
    return ctx.zero

@defun
def conj(ctx, x):
    x = ctx.convert(x)
    try:
        return x.conjugate()
    except AttributeError:
        return x

@defun
def polar(ctx, z):
    return (ctx.fabs(z), ctx.arg(z))

@defun_wrapped
def rect(ctx, r, phi):
    return r * ctx.mpc(*ctx.cos_sin(phi))

@defun
def log(ctx, x, b=None):
    if b is None:
        return ctx.ln(x)
    wp = ctx.prec + 20
    return ctx.ln(x, prec=wp) / ctx.ln(b, prec=wp)

@defun
def log10(ctx, x):
    return ctx.log(x, 10)

@defun
def fmod(ctx, x, y):
    return ctx.convert(x) % ctx.convert(y)

@defun
def degrees(ctx, x):
    return x / ctx.degree

@defun
def radians(ctx, x):
    return x * ctx.degree

def _lambertw_special(ctx, z, k):
    # W(0,0) = 0; all other branches are singular
    if not z:
        if not k:
            return z
        return ctx.ninf + z
    if z == ctx.inf:
        if k == 0:
            return z
        else:
            return z + 2*k*ctx.pi*ctx.j
    if z == ctx.ninf:
        return (-z) + (2*k+1)*ctx.pi*ctx.j
    # Some kind of nan or complex inf/nan?
    return ctx.ln(z)

import math
import cmath

def _lambertw_approx_hybrid(z, k):
    imag_sign = 0
    if hasattr(z, "imag"):
        x = float(z.real)
        y = z.imag
        if y:
            imag_sign = (-1) ** (y < 0)
        y = float(y)
    else:
        x = float(z)
        y = 0.0
        imag_sign = 0
    # hack to work regardless of whether Python supports -0.0
    if not y:
        y = 0.0
    z = complex(x,y)
    if k == 0:
        if -4.0 < y < 4.0 and -1.0 < x < 2.5:
            if imag_sign:
                # Taylor series in upper/lower half-plane
                if y > 1.00: return (0.876+0.645j) + (0.118-0.174j)*(z-(0.75+2.5j))
                if y > 0.25: return (0.505+0.204j) + (0.375-0.132j)*(z-(0.75+0.5j))
                if y < -1.00: return (0.876-0.645j) + (0.118+0.174j)*(z-(0.75-2.5j))
                if y < -0.25: return (0.505-0.204j) + (0.375+0.132j)*(z-(0.75-0.5j))
            # Taylor series near -1
            if x < -0.5:
                if imag_sign >= 0:
                    return (-0.318+1.34j) + (-0.697-0.593j)*(z+1)
                else:
                    return (-0.318-1.34j) + (-0.697+0.593j)*(z+1)
            # return real type
            r = -0.367879441171442
            if (not imag_sign) and x > r:
                z = x
            # Singularity near -1/e
            if x < -0.2:
                return -1 + 2.33164398159712*(z-r)**0.5 - 1.81218788563936*(z-r)
            # Taylor series near 0
            if x < 0.5: return z
            # Simple linear approximation
            return 0.2 + 0.3*z
        if (not imag_sign) and x > 0.0:
            L1 = math.log(x); L2 = math.log(L1)
        else:
            L1 = cmath.log(z); L2 = cmath.log(L1)
    elif k == -1:
        # return real type
        r = -0.367879441171442
        if (not imag_sign) and r < x < 0.0:
            z = x
        if (imag_sign >= 0) and y < 0.1 and -0.6 < x < -0.2:
            return -1 - 2.33164398159712*(z-r)**0.5 - 1.81218788563936*(z-r)
        if (not imag_sign) and -0.2 <= x < 0.0:
            L1 = math.log(-x)
            return L1 - math.log(-L1)
        else:
            if imag_sign == -1 and (not y) and x < 0.0:
                L1 = cmath.log(z) - 3.1415926535897932j
            else:
                L1 = cmath.log(z) - 6.2831853071795865j
            L2 = cmath.log(L1)
    return L1 - L2 + L2/L1 + L2*(L2-2)/(2*L1**2)

def _lambertw_series(ctx, z, k, tol):
    """
    Return rough approximation for W_k(z) from an asymptotic series,
    sufficiently accurate for the Halley iteration to converge to
    the correct value.
    """
    magz = ctx.mag(z)
    if (-10 < magz < 900) and (-1000 < k < 1000):
        # Near the branch point at -1/e
        if magz < 1 and abs(z+0.36787944117144) < 0.05:
            if k == 0 or (k == -1 and ctx._im(z) >= 0) or \
                         (k == 1  and ctx._im(z) < 0):
                delta = ctx.sum_accurately(lambda: [z, ctx.exp(-1)])
                cancellation = -ctx.mag(delta)
                ctx.prec += cancellation
                # Use series given in Corless et al.
                p = ctx.sqrt(2*(ctx.e*z+1))
                ctx.prec -= cancellation
                u = {0:ctx.mpf(-1), 1:ctx.mpf(1)}
                a = {0:ctx.mpf(2), 1:ctx.mpf(-1)}
                if k != 0:
                    p = -p
                s = ctx.zero
                # The series converges, so we could use it directly, but unless
                # *extremely* close, it is better to just use the first few
                # terms to get a good approximation for the iteration
                for l in xrange(max(2,cancellation)):
                    if l not in u:
                        a[l] = ctx.fsum(u[j]*u[l+1-j] for j in xrange(2,l))
                        u[l] = (l-1)*(u[l-2]/2+a[l-2]/4)/(l+1)-a[l]/2-u[l-1]/(l+1)
                    term = u[l] * p**l
                    s += term
                    if ctx.mag(term) < -tol:
                        return s, True
                    l += 1
                ctx.prec += cancellation//2
                return s, False
        if k == 0 or k == -1:
            return _lambertw_approx_hybrid(z, k), False
    if k == 0:
        if magz < -1:
            return z*(1-z), False
        L1 = ctx.ln(z)
        L2 = ctx.ln(L1)
    elif k == -1 and (not ctx._im(z)) and (-0.36787944117144 < ctx._re(z) < 0):
        L1 = ctx.ln(-z)
        return L1 - ctx.ln(-L1), False
    else:
        # This holds both as z -> 0 and z -> inf.
        # Relative error is O(1/log(z)).
        L1 = ctx.ln(z) + 2j*ctx.pi*k
        L2 = ctx.ln(L1)
    return L1 - L2 + L2/L1 + L2*(L2-2)/(2*L1**2), False

@defun
def lambertw(ctx, z, k=0):
    z = ctx.convert(z)
    k = int(k)
    if not ctx.isnormal(z):
        return _lambertw_special(ctx, z, k)
    prec = ctx.prec
    ctx.prec += 20 + ctx.mag(k or 1)
    wp = ctx.prec
    tol = wp - 5
    w, done = _lambertw_series(ctx, z, k, tol)
    if not done:
        # Use Halley iteration to solve w*exp(w) = z
        two = ctx.mpf(2)
        for i in xrange(100):
            ew = ctx.exp(w)
            wew = w*ew
            wewz = wew-z
            wn = w - wewz/(wew+ew-(w+two)*wewz/(two*w+two))
            if ctx.mag(wn-w) <= ctx.mag(wn) - tol:
                w = wn
                break
            else:
                w = wn
        if i == 100:
            ctx.warn("Lambert W iteration failed to converge for z = %s" % z)
    ctx.prec = prec
    return +w

@defun_wrapped
def bell(ctx, n, x=1):
    x = ctx.convert(x)
    if not n:
        if ctx.isnan(x):
            return x
        return type(x)(1)
    if ctx.isinf(x) or ctx.isinf(n) or ctx.isnan(x) or ctx.isnan(n):
        return x**n
    if n == 1: return x
    if n == 2: return x*(x+1)
    if x == 0: return ctx.sincpi(n)
    return _polyexp(ctx, n, x, True) / ctx.exp(x)

def _polyexp(ctx, n, x, extra=False):
    def _terms():
        if extra:
            yield ctx.sincpi(n)
        t = x
        k = 1
        while 1:
            yield k**n * t
            k += 1
            t = t*x/k
    return ctx.sum_accurately(_terms, check_step=4)

@defun_wrapped
def polyexp(ctx, s, z):
    if ctx.isinf(z) or ctx.isinf(s) or ctx.isnan(z) or ctx.isnan(s):
        return z**s
    if z == 0: return z*s
    if s == 0: return ctx.expm1(z)
    if s == 1: return ctx.exp(z)*z
    if s == 2: return ctx.exp(z)*z*(z+1)
    return _polyexp(ctx, s, z)

@defun_wrapped
def cyclotomic(ctx, n, z):
    n = int(n)
    assert n >= 0
    p = ctx.one
    if n == 0:
        return p
    if n == 1:
        return z - p
    if n == 2:
        return z + p
    # Use divisor product representation. Unfortunately, this sometimes
    # includes singularities for roots of unity, which we have to cancel out.
    # Matching zeros/poles pairwise, we have (1-z^a)/(1-z^b) ~ a/b + O(z-1).
    a_prod = 1
    b_prod = 1
    num_zeros = 0
    num_poles = 0
    for d in range(1,n+1):
        if not n % d:
            w = ctx.moebius(n//d)
            # Use powm1 because it is important that we get 0 only
            # if it really is exactly 0
            b = -ctx.powm1(z, d)
            if b:
                p *= b**w
            else:
                if w == 1:
                    a_prod *= d
                    num_zeros += 1
                elif w == -1:
                    b_prod *= d
                    num_poles += 1
    #print n, num_zeros, num_poles
    if num_zeros:
        if num_zeros > num_poles:
            p *= 0
        else:
            p *= a_prod
            p /= b_prod
    return p

@defun
def mangoldt(ctx, n):
    r"""
    Evaluates the von Mangoldt function `\Lambda(n) = \log p`
    if `n = p^k` a power of a prime, and `\Lambda(n) = 0` otherwise.

    **Examples**

        >>> from mpmath import *
        >>> mp.dps = 25; mp.pretty = True
        >>> [mangoldt(n) for n in range(-2,3)]
        [0.0, 0.0, 0.0, 0.0, 0.6931471805599453094172321]
        >>> mangoldt(6)
        0.0
        >>> mangoldt(7)
        1.945910149055313305105353
        >>> mangoldt(8)
        0.6931471805599453094172321
        >>> fsum(mangoldt(n) for n in range(101))
        94.04531122935739224600493
        >>> fsum(mangoldt(n) for n in range(10001))
        10013.39669326311478372032

    """
    n = int(n)
    if n < 2:
        return ctx.zero
    if n % 2 == 0:
        # Must be a power of two
        if n & (n-1) == 0:
            return +ctx.ln2
        else:
            return ctx.zero
    # TODO: the following could be generalized into a perfect
    # power testing function
    # ---
    # Look for a small factor
    for p in (3,5,7,11,13,17,19,23,29,31):
        if not n % p:
            q, r = n // p, 0
            while q > 1:
                q, r = divmod(q, p)
                if r:
                    return ctx.zero
            return ctx.ln(p)
    if ctx.isprime(n):
        return ctx.ln(n)
    # Obviously, we could use arbitrary-precision arithmetic for this...
    if n > 10**30:
        raise NotImplementedError
    k = 2
    while 1:
        p = int(n**(1./k) + 0.5)
        if p < 2:
            return ctx.zero
        if p ** k == n:
            if ctx.isprime(p):
                return ctx.ln(p)
        k += 1




__all__ = ['MUL']

from .base import Head, heads_precedence, Pair, Expr, ArithmeticHead

from ..core import init_module
init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()

class MulHead(ArithmeticHead, Head):
    """
    Algebra(MUL, <list of factors>)

    The list of factors is should not contain numeric expressions
    representing a coefficient, numeric part should be represented
    via TERM_COEFF expression:
    
      Algebra(TERM_COEFF, (Algebra(MUL, <list of factors>), <numeric part>))

    """
    op_mth = '__mul__'
    op_rmth = '__rmul__'

    def is_data_ok(self, cls, data):
        if type(data) in [tuple, list]:
            for a in data:
                if not isinstance(a, cls):
                    return '%s data item must be %s instance but got %s' % (self, cls, type(a))
        else:
            return '%s data part must be a list but got %s' % (self, type(data))

    def __repr__(self): return 'MUL'

    def new(self, cls, operands, evaluate=True):
        operands = [op for op in operands if op!=1]
        n = len(operands)
        if n==0:
            return cls(NUMBER, 1)
        if n==1:
            return operands[0]
        return cls(MUL, operands)

    def reevaluate(self, cls, operands):
        r = operands[0]
        for op in operands[1:]:
            r *= op
        return r

    def base_exp(self, cls, expr):
        return expr, 1

    def data_to_str_and_precedence(self, cls, operands):
        m = len(operands)
        if m==0:
            return '1', heads_precedence.NUMBER
        if m==1:
            op = operands[0]
            return op.head.data_to_str_and_precedence(cls, op.data)
        mul_p = heads_precedence.MUL
        r = ''
        for op in operands:
            f, f_p = op.head.data_to_str_and_precedence(cls, op.data)
            if f=='1': continue
            if not r or r=='-':
                r += '('+f+')' if f_p<mul_p else f
            elif f.startswith('1/'):
                r += '*('+f+')' if f_p<mul_p else f[1:]
            else:
                r += '*('+f+')' if f_p<mul_p else '*'+f
        if not r:
            return '1', heads_precedence.NUMBER
        return r, mul_p

    def term_coeff(self, Algebra, expr):
        data = []
        coeff = 1
        for op in expr.data:
            t, c = op.head.term_coeff(Algebra, op)
            if c is not 1:
                coeff *= c
                if t.head is NUMBER:
                    assert t.data==1,`t`
                else:
                    data.append(t)
            else:
                data.append(op)
        if coeff is 1:
            return expr, 1
        return mul_new(Algebra, data), coeff

    def combine(self, cls, factors_list):
        """ Combine factors in a list and return result.
        """
        lst = []
        compart = 1
        for factor in factors_list:
            if factor.head is NUMBER:
                compart = compart * factor.data
                continue
            r = None
            b2, e2 = factor.head.base_exp(cls, factor)
            if lst and b2.head is MUL:
                l = b2.data
                if len(l)<=len(lst) and lst[-len(l):]==l:
                    # x*a*b*(a*b)**2 -> x*(a*b)**3
                    r = b2 ** (e2 + 1)
                    del lst[-len(l):]
            if lst and r is None:
                b1, e1 = lst[-1].head.base_exp(cls, lst[-1])
                if b1==b2:
                    # x*a**3*a**2 -> x*a**5
                    r = b2 ** (e1 + e2)
                    del lst[-1]
            if r is None:
                lst.append(factor)
                for i in range(2,len(lst)):
                    b1, e1 = lst[-i-1].head.base_exp(cls, lst[-i-1]);
                    if b1.head is MUL:
                        c, l = b1.data
                        if l == lst[-i:] and c==1:
                            # x*(a*b)**2*a*b -> x*(a*b)**3
                            r = b1 ** (e1 + 1)
                            del lst[-i-1:]
                            break
                    if lst[-i:]==lst[-2*i:-i]:
                        # x*a*b * a*b -> x*(a*b)**2
                        r = cls(MUL, lst[-i:])**2
                        del lst[-2*i:]
                        break
            if r is not None:
                if r.head is NUMBER:
                    compart = compart * r
                else:
                    lst.append(r)            
        if not lst:
            return compart
        if compart==1:
            if len(lst)==1:
                return lst[0]
            return cls(MUL, lst)
        return cls(TERM_COEFF, (cls(MUL, lst), compart))

    def non_commutative_mul(self, cls, lhs, rhs):
        head, data = rhs.pair
        if head is NUMBER:
            if data==1:
                return lhs
            return TERM_COEFF.new(cls, (lhs, data))
        if head is SYMBOL or head is POW:
            return self.combine(cls, lhs.data + [rhs])
        if head is TERM_COEFF:
            term, coeff = data
            return (lhs * term) * coeff
        if head is MUL:
            return self.combine(cls, lhs.data + rhs.data)
        raise NotImplementedError(`self, cls, lhs.pair, rhs.pair`)

    def non_commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    non_commutative_rmul_number = non_commutative_mul_number

    def pow(self, cls, base, exp):
        if exp==0:
            return cls(NUMBER, 1)
        if exp==1:
            return base
        if exp==-1:
            factors_list = [factor**-1 for factor in base.data]
            factors_list.reverse()
            return cls(MUL, factors_list)
        term, coeff = self.term_coeff(cls, base)
        if coeff!=1:
            return NUMBER.pow(cls, coeff, exp) * term**exp
        if isinstance(exp, Expr):
            h, d = exp.pair
            if h is NUMBER and isinstance(d, inttypes) and d>0:
                factors_list = base.data
                first = factors_list[0]
                last = factors_list[-1]
                a = last * first
                if a is not None and a.head is NUMBER: # todo: or a is commutative
                    compart = NUMBER.pow_number(cls, a, d)
                    rest = factors_list[1:-1]
                    if not rest:
                        return compart
                    if len(rest)==1:
                        middle = rest[0]
                    else:
                        middle = cls(MUL, rest)
                    return compart * first * middle**d * last # could be optimized
            if h is NUMBER:
                exp = d
        return cls(POW, (base, exp))

    def pow_number(self, cls, base, exp):
        return self.pow(cls, base, cls(NUMBER, exp))

    def walk(self, func, cls, data, target):
        l = []
        flag = False
        for op in data:
            o = op.head.walk(func, cls, op.data, op)
            if op is not o:
                flag = True
            l.append(o)
        if flag:
            r = MUL.new(cls, l)
            return func(cls, r.head, r.data, r)
        return func(cls, self, data, target)

    def scan(self, proc, cls, operands, target):
        for operand in operands:
            operand.head.scan(proc, cls, operand.data, target)
        proc(cls, self, operands, target)

    def combine_mul_list(self, Algebra, data):
        """
        Combine mul operands of an multiplicative group in data.
        data will be changed in place.
        """
        commutative = Algebra.algebra_options.get('is_multiplicative_group_commutative')
        coeff = 1
        if commutative:
            d = {}
            for op in data:
                base, exp = op.head.base_exp(Algebra, op)
                base_exp_dict_add_item(Algebra, d, base, exp)
            data[:] = [pow_new(Algebra, base_exp) for base_exp in d.iteritems()]
        else:
            n = len(data)
            i0 = 0
            while 1:
                i = i0
                if i+1 >= n:
                    break
                lhs = data[i]
                if lhs.head is NUMBER:
                    coeff *= lhs.data
                    del data[i]
                    n -= 1
                    continue
                rhs = data[i+1]
                if rhs.head is NUMBER:
                    coeff *= rhs.data
                    del data[i+1]
                    n -= 1
                    continue
                lbase, lexp = lhs.head.base_exp(Algebra, lhs)
                rbase, rexp = rhs.head.base_exp(Algebra, rhs)
                if lbase==rbase:
                    exp = lexp + rexp
                    if exp:
                        del data[i+1]
                        data[i] = pow_new(Algebra, (lbase, exp))
                        i0 = i
                        n -= 1
                    else:
                        del data[i:i+2]
                        i0 = max(i - 1, 0)
                        n -= 2
                elif not rexp:
                    del data[i+1]
                    n -= 1
                    i0 = i
                elif not lexp:
                    del data[i]
                    n -= 1
                    i0 = max(i-1,0)
                else:
                    i0 += 1
        return coeff

    def to_TERM_COEFF_DICT(self, Algebra, data, expr):
        m = data[0].to(TERM_COEFF_DICT)
        for op in data[1:]:
            m *= op.to(TERM_COEFF_DICT)
        return m

    def to_ADD(self, Algebra, data, expr):
        m = data[0].to(ADD)
        for op in data[1:]:
            m *= op.head.to_ADD(Algebra, op.data, op)
        return m

    def algebra_pos(self, Algebra, expr):
        return expr

    def algebra_neg(self, Algebra, expr):
        if Algebra.algebra_options.get('evaluate_addition'):
            return self.algebra_mul_number(Algebra, expr, -1, False)
        return Algebra(NEG, expr)

    def algebra_add_number(self, Algebra, lhs, rhs, inplace):
        return self.algebra_add(Algebra, lhs, Algebra(NUMBER, rhs), inplace)

    def algebra_add(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('evaluate_addition'):
            rhead, rdata = rhs.pair
            if rhead is TERM_COEFF_DICT or rhead is EXP_COEFF_DICT:
                rhs = rhs.head.to_ADD(Algebra, rdata, rhs)
                rhead, rdata = rhs.pair
            if rhead is ADD:
                data = [lhs] + rdata
            else:
                data = [lhs, rhs]
            ADD.combine_add_list(Algebra, data)
            return add_new(Algebra, data)
        return Algebra(ADD, [lhs, rhs])

    def algebra_mul_number(self, Algebra, lhs, rhs, inplace):
        return self.algebra_mul(Algebra, lhs, Algebra(NUMBER, rhs), inplace)

    def algebra_mul(self, Algebra, lhs, rhs, inplace):
        rhead, rdata = rhs.pair
        if rhead is BASE_EXP_DICT or rhead is TERM_COEFF:
            rhs = rhs.to(MUL)
            rhead, rdata = rhs.pair
        if inplace:
            data = lhs.data
        else:
            data = lhs.data[:]
        if rhead is MUL:
            data.extend(rdata)
        else:
            data.append(rhs)
        if Algebra.algebra_options.get('evaluate_multiplication'):
            coeff = self.combine_mul_list(Algebra, data)
            if not coeff:
                return Algebra(NUMBER, 0)
            if coeff != 1:
                if len(data)==1:
                    return Algebra(TERM_COEFF, (data[0], coeff))
                data.insert(0, Algebra(NUMBER, coeff))
        if inplace:
            return mul(Algebra, lhs)
        return mul_new(Algebra, data)

    def algebra_pow_number(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('evaluate_multiplication'):
            if rhs==1:
                return lhs
            if not rhs:
                return Algebra(NUMBER, 1)
            if rhs==-1:
                return Algebra(MUL, [op**-1 for op in lhs.data[::-1]])
        return super(type(self), self).algebra_pow_number(Algebra, lhs, rhs, inplace)

MUL = MulHead()



'''
.. module:: pymaclab
   :platform: Linux
   :synopsis: The var module contains the VAR class for estimating and doing further work with Vector Autoregressions commonly
              used in applied macroeconometrics. It supports advanced methods such as bootstrapping confidence intervals including
              Killian's boostrap-after-bootstrap small-sample bias correction. Also CPU-intensive methods such as the bootstrap can
              be computed using Parallel Python to exploit multi-core CPUs. Pretty plotting methods are also included which depend
              on matplotlib.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''

import os as OPS
from dsge import macrolab
from stats import var
from stats import favar
import linalg
import sys
import pp
from copy import deepcopy

# Expose the version number into library root
from version import version as __version__

# Start the job_server
ppservers = ()
jobserver = pp.Server(ppservers=ppservers)
ncpus = jobserver.get_ncpus()
del jobserver
del ppservers
print "PyMacLab started with parallel jobserver using "+str(ncpus)+" cores..."


# Some helper functions
def db_graph(dbase,tseries):
    fig = TPL.tsfigure()
    fsp = fig.add_tsplot(111)
    P.title(dbase.datdic[tseries]['Dat_Desc'])
    fsp.tsplot(dbase.datdic[tseries]['infile'], '-', label=tseries)
    fsp.format_dateaxis()
    fsp.set_xlim(int(dbase.datdic[tseries]['infile'].start_date),
                 int(dbase.datdic[tseries]['infile'].end_date))
    fsp.legend()
    P.show()

def newMOD(txtfile=None,dbase=None,initlev=3,mesg=False,ncpus=ncpus,\
           mk_hessian=True,use_focs=False,ssidic=None,sstate=None,vtiming={'exo':[-1,0],'endo':[-1,0],'con':[0,1]}):
    '''
    Model's second intialisation method called by newMOD() function call. The model's
    __init__() method only creates the instance and adds information, but does no
    parsing and computations whatsoever. init2() parses and ready's the model for
    calculating the steady state and the dynamic solution to the model.

    init_lev = 0: only parsing, no calculations, but preparing for manual SS solution
    init_lev = 1: parsing and steady state calculations
    init_lev = 2: parsing, steady state calculations and dynamic solution computation
    '''
    # Do error handling for vtiming
    for keyo in vtiming.keys():
        if abs(vtiming[keyo][0]-vtiming[keyo][1]) != 1:
            print "ERROR: Periods defined in vtiming have to be adjacent, check ", keyo
            sys.exit()
    # Create a new vtiming so that users can pass only one or two keys at instantiation of model
    # Clearly then vtiming2 represents the standard values which can get updated using vtiming
    vtiming2={'exo':[-1,0],'endo':[-1,0],'con':[0,1]}
    vtiming2.update(vtiming)
    modobj = macrolab.DSGEmodel(txtfile,dbase=dbase,initlev=initlev,mesg=mesg,ncpus=ncpus,\
                                mk_hessian=mk_hessian,use_focs=use_focs,\
                                ssidic=ssidic,sstate=sstate,vtiming=vtiming2)
    modobj.inits.init1()
    modobj.inits.init1a()
    modobj.inits.init1b()
    modobj.inits.init1c()
    modobj.inits.init2()
    # Ready to solve for SS manually at command prompt
    if initlev == 0:
        modobj.inits.init_out()
        return modobj
    # SS solved automatically, but only if SS has not been handed over entirely externally
    if sstate == None:
        modobj.inits.init3()
    else:
        if mesg: print "SS: Using entirely EXTERNALLY supplied steady state values dictionary..."
        modobj.sstate = deepcopy(modobj._sstate)
    if initlev == 1:
        modobj.inits.init_out()
        return modobj
    # Dynamic system prepared and prepared for solving
    modobj.inits.init4()
    if initlev == 2:
        modobj.inits.init_out()
        return modobj
    modobj.inits.init5()
    if initlev == 3:
        modobj.inits.init_out()
        return modobj

def newDB():
    return macrolab.TSDataBase()

def newVAR(data=None,vnames=None,pnames=None,svnames=None,irfs=True,boot=True,plot=True,conf=None,mesg=False):
    return var.VAR(data=data,vnames=vnames,pnames=pnames,svnames=svnames,irfs=irfs,boot=boot,plot=plot,conf=conf,mesg=mesg)

def newFAVAR(dates=None,rdates=None,data=None,freq='M',vnames=None,pnames=None,svnames=None,irfs=True,rescale=False,boot=True,plot=True,sfacs='auto',init=None,conf=None,mesg=False):
    return favar.FAVAR(dates=dates,rdates=rdates,data=data,freq=freq,vnames=vnames,pnames=pnames,svnames=svnames,irfs=irfs,rescale=rescale,boot=boot,plot=plot,sfacs=sfacs,init=init,conf=conf,mesg=mesg)

def modinfo(model):
    offset = 40
    # Print the Description
    print '\n'
    print'Model Description:'
    print 60*'-'
    for x in model.txtpars.secs['mod'][1]:
        print x
    print 60*'-'
    # What's it's name?
    print 'Model Name:'+(offset-len('Model Name:'))*' '+model.modname
    # From which File parsed?
    print 'Created from:'+(offset-len('Created from:'))*' '+model.modfile
    # Supplied with data?
    if 'data' in dir(model) and model.data != None:
        print 'Data attached?'+(offset-len('Data attached?'))*' '+'YES'
    else:
        print 'Data attached?'+(offset-len('Data attached?'))*' '+'NO'
    # Possesses matuhlig?
    if 'matuhlig' in dir(model.modsolvers):
        # Possesses matuhlig solution?
        if 'PP' in dir(model.modsolvers.matuhlig):
            print 'MatUhlig Solution available?'+(offset-len('MatUhlig Solution available?'))*' '+'YES(*)'
        else:
            print 'MatUhlig Solution available?'+(offset-len('MatUhlig Solution available?'))*' '+'NO(*)'
    else:
        print 'MatUhlig Solution available?'+(offset-len('MatUhlig Solution available?'))*' '+'NO(X)'
    # Possesses pyuhlig?
    if 'pyuhlig' in dir(model.modsolvers):
        # Possesses pyuhlig solution?
        if 'PP' in dir(model.modsolvers.pyuhlig):
            print 'PyUhlig Solution available?'+(offset-len('PyUhlig Solution available?'))*' '+'YES(*)'
        else:
            print 'PyUhlig Solution available?'+(offset-len('PyUhlig Solution available?'))*' '+'NO(*)'
    else:
        print 'PyUhlig Solution available?'+(offset-len('PyUhlig Solution available?'))*' '+'NO(X)'
    # Possesses matklein?
    if 'matklein' in dir(model.modsolvers):
        # Possesses matklein solution?
        if 'P' in dir(model.modsolvers.matklein):
            print 'MatKlein Solution available?'+(offset-len('MatKlein Solution available?'))*' '+'YES(*)'
        else:
            print 'MatKlein Solution available?'+(offset-len('MatKlein Solution available?'))*' '+'NO(*)'
    else:
        print 'MatKlein Solution available?'+(offset-len('MatKlein Solution available?'))*' '+'NO(X)'
    # Possesses forklein?
    if 'forklein' in dir(model.modsolvers):
        # Possesses forklein solution?
        if 'P' in dir(model.modsolvers.forklein):
            print 'ForKlein Solution available?'+(offset-len('ForKlein Solution available?'))*' '+'YES(*)'
        else:
            print 'ForKlein Solution available?'+(offset-len('ForKlein Solution available?'))*' '+'NO(*)'
    else:
        print 'ForKlein Solution available?'+(offset-len('ForKlein Solution available?'))*' '+'NO(X)'
    # Possesses forkleind?
    if 'forkleind' in dir(model.modsolvers):
        # Possesses forkleind solution?
        if 'P' in dir(model.modsolvers.forkleind):
            print 'ForKleinD Solution available?'+(offset-len('ForKleinD Solution available?'))*' '+'YES(*)'
        else:
            print 'ForKleinD Solution available?'+(offset-len('ForKleinD Solution available?'))*' '+'NO(*)'
    else:
        print 'ForKleinD Solution available?'+(offset-len('ForKleinD Solution available?'))*' '+'NO(X)'
    # Possesses matklein2d?
    if 'matklein2d' in dir(model.modsolvers):
        # Possesses matklein2d solution?
        if 'P' in dir(model.modsolvers.matklein2d):
            print 'MatKlein2D Solution available?'+(offset-len('MatKlein2D Solution available?'))*' '+'YES(*)'
        else:
            print 'MatKlein2D Solution available?'+(offset-len('MatKlein2D Solution available?'))*' '+'NO(*)'
    else:
        print 'MatKlein2D Solution available?'+(offset-len('MatKlein2D Solution available?'))*' '+'NO(X)'

def modsolve(model,stype):
    if stype == 'matuhlig':
        if 'PP' in dir(model.modsolvers.matuhlig):
            del model.modsolvers.matuhlig.PP
            del model.modsolvers.matuhlig.QQ
            del model.modsolvers.matuhlig.RR
            del model.modsolvers.matuhlig.SS
            del model.modsolvers.matuhlig.WW
        model.modsolvers.matuhlig.solve()
        if 'PP' in dir(model.modsolvers.matuhlig):
            return dict(PP=model.modsolvers.matuhlig.PP,
                        QQ=model.modsolvers.matuhlig.QQ,
                        RR=model.modsolvers.matuhlig.RR,
                        SS=model.modsolvers.matuhlig.SS,
                        WW=model.modsolvers.matuhlig.WW)
        else:
            print 'Error: Solution not obtained!'
            return False
    elif stype == 'matklein':
        if 'P' in dir(model.modsolvers.matklein):
            del model.modsolvers.matklein.P
            del model.modsolvers.matklein.F
        model.modsolvers.matklein.solve()
        if 'P' in dir(model.modsolvers.matklein):
            return dict(P=model.modsolvers.matklein.P,
                        F=model.modsolvers.matklein.F)
        else:
            print 'Error: Solution not obtained!'
            return False
    elif stype == 'pyuhlig':
        if 'PP' in dir(model.modsolvers.pyuhlig):
            del model.modsolvers.pyuhlig.PP
            del model.modsolvers.pyuhlig.QQ
            del model.modsolvers.pyuhlig.RR
            del model.modsolvers.pyuhlig.SS
            del model.modsolvers.pyuhlig.WW
        model.modsolvers.pyuhlig.solve()
        if 'PP' in dir(model.modsolvers.pyuhlig):
            return dict(PP=model.modsolvers.pyuhlig.PP,
                        QQ=model.modsolvers.pyuhlig.QQ,
                        RR=model.modsolvers.pyuhlig.RR,
                        SS=model.modsolvers.pyuhlig.SS,
                        WW=model.modsolvers.pyuhlig.WW)
        else:
            print 'Error: Solution not obtained!'
            return False
    elif stype == 'forklein':
        if 'P' in dir(model.modsolvers.forklein):
            del model.modsolvers.forklein.P
            del model.modsolvers.forklein.F
        model.modsolvers.forklein.solve()
        if 'P' in dir(model.modsolvers.forklein):
            return dict(P=model.modsolvers.forklein.P,
                        F=model.modsolvers.forklein.F)
        else:
            print 'Error: Solution not obtained!'
            return False
    else:
        return 'Solution method not recognized!'

def lmods():
    aa = [x[0] for x in filter(lambda x: isinstance(x[1],macrolab.DSGEmodel),macrolab.locdic.items())]
    if len(aa) == 0:
        return 'You have no maclab.DSGE models in memory yet!'
    aa.sort()
    tabu = 5
    # Find longest model name
    mlen = 0
    for x in aa:
        if len(x) > mlen:
            mlen = len(x)
    # Find longest model description
    mdlen = 0
    for x in aa:
        if len(macrolab.locdic[x].modname) > mdlen:
            mdlen = len(macrolab.locdic[x].modname)
    # Find longest model var number
    mvlen = 0
    for x in aa:
        if len(str(macrolab.locdic[x].nall)) > mvlen:
            mvlen = len(str(macrolab.locdic[x].nall))
    # Which is bigger? #Vars or mvlen?
    if len('#Vars') > mvlen:
        mvlen = len('#Vars')

    print ''
    print 'Name'+(mlen+tabu-len('Name'))*' '+'Description'+(mdlen+tabu-len('Description'))*' '+'#Vars'
    print '-'*(mlen+mdlen+mvlen+2*tabu)
    for x in aa:
        print x+(mlen+tabu-len(x))*' '+macrolab.locdic[x].modname+(mdlen+tabu-len(macrolab.locdic[x].modname))*' '+str(macrolab.locdic[x].nall)

def ldbs():
    aa = [x[0] for x in filter(lambda x: isinstance(x[1],macrolab.TSDataBase),macrolab.locdic.items())]
    if len(aa) == 0:
        return 'You have no maclab.DB databases in memory yet!'
    aa.sort()
    return aa

def lvars():
    aa = [x[0] for x in filter(lambda x: isinstance(x[1],macrolab.VAR),macrolab.locdic.items())]
    if len(aa) == 0:
        return 'You have no maclab.VAR vector autoregressions in memory yet!'
    aa.sort()
    return aa

def pyed(pyfile=None):
    cwd = OPS.getcwd()
    dirli = OPS.listdir(cwd)
    if pyfile == None:
        return 'You did not specify the Python file to edit!'
    elif pyfile+'.py' in dirli:
        cmd = macrolab.pyedpath+' '+pyfile+'.py'
        OPS.system(cmd)
    else:
        return 'The file '+pyfile+'.py '+'was not found in current directory!'

def modedit(model):
    model.txted()

def texedit(model):
    model.texed()

def explain(model):
    model.pdf()

############ Make nost tests available###############
def test():
    try:
        import nose
        retv = nose.run(defaultTest='pymaclab.tests')
        return retv
    except:
        print "You need to install nose to run the tests."
        print 'Run "sudo pip install nose" to install it.'

#####################################################



# Import this late because cascading imports need newMOD() function
import modfiles
from .modfiles.makemod import make_modfile



from sympycore.calculus import *

def test_factorial():
    assert isinstance(factorial(2), Calculus)
    assert factorial(2)==2
    assert factorial(3)==6
    assert factorial(4)==24
    assert factorial(1)==1
    assert factorial(0)==1
    
    




#obsolete, each head should be defined in a separate file

__all__ = ['FLOORDIV', 'MOD']


from .base import NaryHead

class ModHead(NaryHead):

    """
    ModHead represents module n-ary operation,
    data is a n-tuple of expression operands.
    """
    op_mth = '__mod__'
    op_rmth = '__rmod__'
    op_symbol = '%'
    def __repr__(self): return 'MOD'

class FloordivHead(NaryHead):
    """
    FloordivHead represents floor-division n-ary operation,
    data is a n-tuple of expression operands.
    """
    op_mth = '__floordiv__'
    op_rmth = '__rfloordiv__'
    op_symbol = '//'    

    def __repr__(self): return 'FLOORDIV'



MOD = ModHead()

FLOORDIV = FloordivHead()





from sympycore import CollectingField as Algebra

Symbol = Algebra.Symbol
Number = Algebra.Number
Add = Algebra.Add
Mul = Algebra.Mul
Pow = Algebra.Pow
Terms = Algebra.Terms
Factors = Algebra.Factors
Apply = Algebra.Apply

x,y,z = map(Symbol, 'xyz')

def foo(x):
    """ Python function can be used to construct applied functions.
    """
    if x==0:
        return x
    return Apply(foo, x)

class bar(object):
    """ Python new-style class can be used to construct applied
    functions.
    """
    def __new__(cls, x):
        if x==0:
            return x
        return Apply(cls, x)

class Fun:
    """ A callable class instance can be used to construct applied
    functions.
    """
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name
    
    def __call__(self, x):
        if x==0:
            return x
        return Apply(self, x)

fun = Fun('fun')

def mysin(x):
    if x==0:
        return x
    return Apply(mysin, x)

def mycos(x):
    if x==0:
        return Algebra.one
    return Apply(mycos, x)

mysin.derivative = lambda arg: mycos(arg)
mycos.derivative = lambda arg: -mysin(arg)

def foo2(x, y):
    if x==y:
        return Algebra.one
    return Apply(foo2, x,y)

def foo2_1(x, y):
    return Apply(foo2_1, x,y)
def foo2_2(x, y):
    return Apply(foo2_2, x,y)

foo2.derivative = foo2_1,foo2_2

def bar2(x, y):
    return Apply(bar2, x,y)

def test_foo():
    assert str(foo(x))=='foo(x)',str(foo(x))
    assert str(foo(0))=='0'
    assert str(foo(x+y)) in ['foo(x + y)', 'foo(y + x)'], str(foo(x+y))

    assert foo(x)==foo(x)
    assert foo(x).subs(x,z)==foo(z), `foo(x).subs(x,z), foo(z)`
    assert foo(x).subs(x,0)==0, `foo(x).subs(x,0)`
    assert (y+foo(x)).subs(x,0)==y, `(y+foo(x)).subs(x,0)`

    assert str(foo(foo(x)))=='foo(foo(x))'
    assert str(foo(x)+foo(y))=='foo(x) + foo(y)', str(foo(x)+foo(y))
    assert str(foo(x)+x*foo(y)) in ['foo(x) + x*foo(y)', 'x*foo(y) + foo(x)'], str(foo(x)+x*foo(y))

    assert str((foo(x)+foo(y))**2)=='(foo(x) + foo(y))**2'
    assert str(((foo(x)+foo(y))**2).expand()) in ['2*foo(x)*foo(y) + foo(x)**2 + foo(y)**2',
                                                  'foo(x)**2 + foo(y)**2 + 2*foo(x)*foo(y)',
                                                  'foo(y)**2 + foo(x)**2 + 2*foo(x)*foo(y)',
                                                  'foo(x)**2 + 2*foo(x)*foo(y) + foo(y)**2',
                                                  'foo(y)**2 + 2*foo(x)*foo(y) + foo(x)**2'], str(((foo(x)+foo(y))**2).expand())


def test_bar():
    assert str(bar(x))=='bar(x)'
    assert str(bar(0))=='0'
    assert str(bar(x+y)) in ['bar(x + y)',
                             'bar(y + x)',
                             ], str(bar(x+y))

    assert bar(x)==bar(x)
    assert bar(x).subs(x,z)==bar(z)
    assert bar(x).subs(x,0)==0
    assert (y+bar(x)).subs(x,0)==y

    assert str(bar(bar(x)))=='bar(bar(x))'
    assert str(bar(x)+bar(y))=='bar(x) + bar(y)'
    assert str(bar(x)+x*bar(y)) in ['bar(x) + x*bar(y)', 'x*bar(y) + bar(x)'], str(bar(x)+x*bar(y))

    assert str(((bar(x)+bar(y))**2).expand()) in ['2*bar(x)*bar(y) + bar(x)**2 + bar(y)**2',
                                                  'bar(x)**2 + bar(y)**2 + 2*bar(x)*bar(y)',
                                                  'bar(y)**2 + bar(x)**2 + 2*bar(x)*bar(y)',
                                                  'bar(x)**2 + 2*bar(x)*bar(y) + bar(y)**2',
                                                  'bar(y)**2 + 2*bar(x)*bar(y) + bar(x)**2'], str(((bar(x)+bar(y))**2).expand())

def test_fun():
    assert str(fun(x))=='fun(x)',str(fun(x))
    assert str(fun(0))=='0'
    assert str(fun(x+y)) in ['fun(x + y)', 'fun(y + x)'], str(fun(x+y))

    assert fun(x)==fun(x)
    assert fun(x).subs(x,z)==fun(z)
    assert fun(x).subs(x,0)==0
    assert (y+fun(x)).subs(x,0)==y

    assert str(fun(fun(x)))=='fun(fun(x))'
    assert str(fun(x)+fun(y))=='fun(x) + fun(y)'
    assert str(fun(x)+x*fun(y)) in ['fun(x) + x*fun(y)', 'x*fun(y) + fun(x)'],str(fun(x)+x*fun(y))

    assert str(((fun(x)+fun(y))**2).expand()) in ['2*fun(x)*fun(y) + fun(x)**2 + fun(y)**2',
                                                  'fun(x)**2 + fun(y)**2 + 2*fun(x)*fun(y)',
                                                  'fun(y)**2 + fun(x)**2 + 2*fun(x)*fun(y)',
                                                  'fun(x)**2 + 2*fun(x)*fun(y) + fun(y)**2',
                                                  'fun(y)**2 + 2*fun(x)*fun(y) + fun(x)**2'], str(((fun(x)+fun(y))**2).expand())
    

def test_foo2():
    assert str(foo2(x,x))=='1'
    assert str(foo2(x,y))=='foo2(x, y)'
    assert str(foo2(x,y).subs(x,z))=='foo2(z, y)'

def test_bar2():
    assert str(bar2(x,y))=='bar2(x, y)'
    assert str(bar2(x,y).subs(x,z))=='bar2(z, y)'



"""Generated low-level arithmetic methods for CollectingField.

This file is generated by the src/mk_pairs_ops.py script.
See http://sympycore.googlecode.com/ for more information.

DO NOT CHANGE THIS FILE DIRECTLY!!!
"""

from ..core import Expr
from ..utils import NUMBER, SYMBOL, TERMS
from ..heads import BASE_EXP_DICT as FACTORS
from ..arithmetic.numbers import (normalized_fraction, mpq, try_power,
  numbertypes, inttypes_set, numbertypes_set)
from ..arithmetic.infinity import Infinity

def div(a, b, cls):
    if type(b) in inttypes_set:
        if type(a) in inttypes_set:
            if not b:
                if not a:
                    return cls.undefined
                return cls.zoo
            return normalized_fraction(a, b)
        if not b:
            return a * cls.zoo
        if b == 1:
            return a
        return a * mpq((1,b))
    return a / b




def expand_mul_method(cls, self, other):
    lhead, ldata = self.pair
    rhead, rdata = other.pair
    if lhead is FACTORS:
        if rhead is FACTORS:
            #MUL_FACTORS_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = ldata.copy()
            pairs_get = pairs.get
            number = 1
            for t,c in rdata.iteritems():
                #MUL_FACTOR_VALUE_DICT(FACTOR=t; SIGN=+; USIGN=; VALUE=c; DICT=pairs; DICT_GET=pairs_get; NUMBER=number)
                _tmp9 = pairs_get(t)
                if _tmp9 is None:
                    pairs[t] =  c
                else:
                    _tmp9 = _tmp9 + c
                    if type(_tmp9) is cls and _tmp9.head is NUMBER:
                        _tmp9 = _tmp9.data
                    if _tmp9:
                        if t.head is NUMBER:
                            del pairs[t]
                            z, sym = try_power(t.data, _tmp9)
                            if sym:
                                for t1, c1 in sym:
                                    #NEWINSTANCE(OBJ=tt; HEAD=NUMBER; DATA=t1)
                                    tt = cls(NUMBER, t1)
                                    #ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=tt; VALUE=c1; SIGN=+; USIGN=)
                                    _tmp23 = pairs_get(tt)
                                    if _tmp23 is None:
                                        pairs[tt] =  c1
                                    else:
                                        _tmp23 = _tmp23 + c1
                                        if _tmp23:
                                            pairs[tt] = _tmp23
                                        else:
                                            del pairs[tt]
                            number = number * z
                        else:
                            pairs[t] = _tmp9
                    else:
                        del pairs[t]
            #CANONIZE_FACTORS_DICT(DICT=pairs; NUMBER=number)
            if not pairs:
                if number is 1:
                    return cls.one
                return number
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   if number==1:
                       return t
                   #RETURN_NEW(HEAD=TERMS; DATA={t: number})
                   #NEWINSTANCE(OBJ=_tmp37; HEAD=TERMS; DATA={t: number})
                   _tmp37 = cls(TERMS, {t: number})
                   return _tmp37
               if t==cls.one:
                   return number
            if number == 1:
                #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
                #NEWINSTANCE(OBJ=_tmp51; HEAD=FACTORS; DATA=pairs)
                _tmp51 = cls(FACTORS, pairs)
                return _tmp51
            #NEWINSTANCE(OBJ=obj; HEAD=FACTORS; DATA=pairs)
            obj = cls(FACTORS, pairs)
            #RETURN_NEW(HEAD=TERMS; DATA={obj: number})
            #NEWINSTANCE(OBJ=_tmp72; HEAD=TERMS; DATA={obj: number})
            _tmp72 = cls(TERMS, {obj: number})
            return _tmp72
        elif rhead is NUMBER:
            return self # other must be one
        else:
            #MUL_FACTORS_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = ldata.copy()
            #MUL_DICT_SYMBOL(DICT=pairs; RHS=other; RHSDATA=rdata)
            #ADD_TERM_VALUE_DICT(TERM=other; VALUE=1; DICT=pairs; DICT_GET=pairs.get; SIGN=+; USIGN=)
            _tmp100 = pairs.get(other)
            if _tmp100 is None:
                pairs[other] =  1
            else:
                _tmp100 = _tmp100 + 1
                if _tmp100:
                    pairs[other] = _tmp100
                else:
                    del pairs[other]
            #CANONIZE_FACTORS_DICT1(DICT=pairs)
            if not pairs:
                return cls.one
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   return t
               if t==cls.one:
                   return t
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp114; HEAD=FACTORS; DATA=pairs)
            _tmp114 = cls(FACTORS, pairs)
            return _tmp114
    elif lhead is NUMBER:
        return other # self must be one
    else:
        if rhead is FACTORS:
            #MUL_SYMBOL_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #MUL_FACTORS_SYMBOL(LHS=other; LHSDATA=rdata; RHS=self; RHSDATA=ldata)
            pairs = rdata.copy()
            #MUL_DICT_SYMBOL(DICT=pairs; RHS=self; RHSDATA=ldata)
            #ADD_TERM_VALUE_DICT(TERM=self; VALUE=1; DICT=pairs; DICT_GET=pairs.get; SIGN=+; USIGN=)
            _tmp149 = pairs.get(self)
            if _tmp149 is None:
                pairs[self] =  1
            else:
                _tmp149 = _tmp149 + 1
                if _tmp149:
                    pairs[self] = _tmp149
                else:
                    del pairs[self]
            #CANONIZE_FACTORS_DICT1(DICT=pairs)
            if not pairs:
                return cls.one
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   return t
               if t==cls.one:
                   return t
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp163; HEAD=FACTORS; DATA=pairs)
            _tmp163 = cls(FACTORS, pairs)
            return _tmp163
        elif rhead is NUMBER:
            return self # other must be one
        else:
            #MUL_SYMBOL_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            if self == other:
                pairs = {self: 2}
            else:
                pairs = {self: 1, other: 1}
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp184; HEAD=FACTORS; DATA=pairs)
            _tmp184 = cls(FACTORS, pairs)
            return _tmp184

#def rsub_method(self, other, NUMBER=NUMBER, TERMS=TERMS, FACTORS=FACTORS):
#    cls = type(self)
#    lhead, ldata = self.pair
#    if isinstance(other, cls.coefftypes):
#        if lhead is NUMBER:
#            @SUB_VALUE_NUMBER(VALUE=other; RHS=self; RHSDATA=ldata)
#        elif lhead is TERMS:
#            @SUB_VALUE_TERMS(VALUE=other; RHS=self; RHSDATA=ldata)
#        else:
#            @SUB_VALUE_SYMBOL(VALUE=other; RHS=self; RHSDATA=ldata)
#    other = cls.convert(other, False)
#    if other is NotImplemented:
#        return other
#    return other - self

def rdiv_method(self, other, NUMBER=NUMBER, TERMS=TERMS, FACTORS=FACTORS):
    cls = type(self)
    lhead, ldata = self.pair
    if isinstance(other, cls.coefftypes):
        if lhead is NUMBER:
            #DIV_VALUE_NUMBER(VALUE=other; RHS=self; RHSDATA=ldata)
            #RETURN_NEW2(HEAD=NUMBER; DATA=div(other, ldata, cls))
            _tmp205 = div(other, ldata, cls)
            if isinstance(_tmp205, Infinity):
                return _tmp205
            #RETURN_NEW(HEAD=NUMBER; DATA=_tmp205)
            #NEWINSTANCE(OBJ=_tmp212; HEAD=NUMBER; DATA=_tmp205)
            _tmp212 = cls(NUMBER, _tmp205)
            return _tmp212
        elif lhead is TERMS:
            #DIV_VALUE_TERMS(VALUE=other; RHS=self; RHSDATA=ldata)
            _tmp226 = other
            if not _tmp226:
                return cls.zero
            pairs = ldata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                c = div(other, c, cls)
                t = 1/t
                if c==1:
                    return t
                if t==cls.one:
                    return cls(NUMBER, c)
                #RETURN_NEW(HEAD=TERMS; DATA={t: c})
                #NEWINSTANCE(OBJ=_tmp233; HEAD=TERMS; DATA={t: c})
                _tmp233 = cls(TERMS, {t: c})
                return _tmp233
            #NEWINSTANCE(OBJ=_tmp226; HEAD=FACTORS; DATA={self: -1})
            _tmp226 = cls(FACTORS, {self: -1})
            if other==1:
                return _tmp226
            #RETURN_NEW(HEAD=TERMS; DATA={_tmp226: other})
            #NEWINSTANCE(OBJ=_tmp254; HEAD=TERMS; DATA={_tmp226: other})
            _tmp254 = cls(TERMS, {_tmp226: other})
            return _tmp254
        elif lhead is FACTORS:
            #DIV_VALUE_FACTORS(VALUE=other; RHS=self; RHSDATA=ldata)
            
            pairs = ldata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                c = -c
                if c==1:
                    return t * other
                new_pairs = {t: c}
            else:
                #NEG_DICT_VALUES(DICT_IN=pairs; DICT_OUT=new_pairs)
                new_pairs = dict([(t, -c) for t,c in pairs.iteritems()])
            #NEWINSTANCE(OBJ=_tmp268; HEAD=FACTORS; DATA=new_pairs)
            _tmp268 = cls(FACTORS, new_pairs)
            if other==1:
                return _tmp268
            #RETURN_NEW(HEAD=TERMS; DATA={_tmp268: other})
            #NEWINSTANCE(OBJ=_tmp289; HEAD=TERMS; DATA={_tmp268: other})
            _tmp289 = cls(TERMS, {_tmp268: other})
            return _tmp289
        else:
            #DIV_VALUE_SYMBOL(VALUE=other; RHS=self; RHSDATA=ldata)
            _tmp303 = other
            if not _tmp303:
                return cls.zero
            #NEWINSTANCE(OBJ=obj2; HEAD=FACTORS; DATA={self: -1})
            obj2 = cls(FACTORS, {self: -1})
            if _tmp303==1:
                return obj2
            #RETURN_NEW(HEAD=TERMS; DATA={obj2: _tmp303})
            #NEWINSTANCE(OBJ=_tmp317; HEAD=TERMS; DATA={obj2: _tmp303})
            _tmp317 = cls(TERMS, {obj2: _tmp303})
            return _tmp317
    other = cls.convert(other, False)
    if other is NotImplemented:
        return other
    return other / self


def div_method(self, other, NUMBER=NUMBER, TERMS=TERMS, FACTORS=FACTORS):
    cls = type(self)
    lhead, ldata = self.pair
    if type(other) is not cls:
        if isinstance(other, cls.coefftypes):
            if lhead is NUMBER:
                #DIV_NUMBER_VALUE(VALUE=other; LHS=self; LHSDATA=ldata)
                #RETURN_NEW2(HEAD=NUMBER; DATA=div(ldata, other, cls))
                _tmp338 = div(ldata, other, cls)
                if isinstance(_tmp338, Infinity):
                    return _tmp338
                #RETURN_NEW(HEAD=NUMBER; DATA=_tmp338)
                #NEWINSTANCE(OBJ=_tmp345; HEAD=NUMBER; DATA=_tmp338)
                _tmp345 = cls(NUMBER, _tmp338)
                return _tmp345
            elif lhead is TERMS:
                #DIV_TERMS_VALUE(VALUE=other; LHS=self; LHSDATA=ldata)
                #MUL_TERMS_VALUE(LHS=self; LHSDATA=ldata; VALUE=div(1,other,cls))
                #MUL_VALUE_TERMS(VALUE=div(1,other,cls); RHS=self; RHSDATA=ldata)
                _tmp373 = div(1,other,cls)
                if not _tmp373:
                    return cls.zero
                pairs = ldata
                if len(pairs)==1:
                    t, c = pairs.items()[0]
                    c = _tmp373 * c
                    if c==1:
                        return t
                    #RETURN_NEW(HEAD=TERMS; DATA={t: c})
                    #NEWINSTANCE(OBJ=_tmp380; HEAD=TERMS; DATA={t: c})
                    _tmp380 = cls(TERMS, {t: c})
                    return _tmp380
                if _tmp373==1:
                    return self
                pairs = {}
                for t,c in ldata.iteritems():
                    pairs[t] = _tmp373 * c
                #NEWINSTANCE(OBJ=obj; HEAD=TERMS; DATA=pairs)
                obj = cls(TERMS, pairs)
                coeff, terms = self._coeff_terms
                if terms is not None:
                    c = coeff * _tmp373
                    if not c==1:
                        obj._coeff_terms = (c, terms)
                else:
                    obj._coeff_terms = (_tmp373, self)
                return obj
            elif lhead is FACTORS:
                #DIV_FACTORS_VALUE(VALUE=other; LHS=self; LHSDATA=ldata)
                #MUL_FACTORS_VALUE(LHS=self; LHSDATA=ldata; VALUE=div(1,other,cls))
                #MUL_SYMBOL_VALUE(VALUE=div(1,other,cls); LHS=self; LHSDATA=ldata)
                #MUL_VALUE_SYMBOL(VALUE=div(1,other,cls); RHS=self; RHSDATA=ldata)
                _tmp422 = div(1,other,cls)
                if not _tmp422:
                    return cls.zero
                if _tmp422==1:
                    return self
                #RETURN_NEW(HEAD=TERMS; DATA={self: _tmp422})
                #NEWINSTANCE(OBJ=_tmp429; HEAD=TERMS; DATA={self: _tmp422})
                _tmp429 = cls(TERMS, {self: _tmp422})
                return _tmp429
            else:
                #DIV_SYMBOL_VALUE(VALUE=other; LHS=self; LHSDATA=ldata)
                #MUL_VALUE_SYMBOL(VALUE=div(1, other, cls); RHS=self; RHSDATA=ldata)
                _tmp450 = div(1, other, cls)
                if not _tmp450:
                    return cls.zero
                if _tmp450==1:
                    return self
                #RETURN_NEW(HEAD=TERMS; DATA={self: _tmp450})
                #NEWINSTANCE(OBJ=_tmp457; HEAD=TERMS; DATA={self: _tmp450})
                _tmp457 = cls(TERMS, {self: _tmp450})
                return _tmp457
        other = cls.convert(other, False)
        if other is NotImplemented:
            return other
    rhead, rdata = other.pair
    if lhead is NUMBER:
        if rhead is NUMBER:
            #DIV_NUMBER_NUMBER(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_VALUE_NUMBER(VALUE=ldata; RHS=other; RHSDATA=rdata)
            #RETURN_NEW2(HEAD=NUMBER; DATA=div(ldata, rdata, cls))
            _tmp485 = div(ldata, rdata, cls)
            if isinstance(_tmp485, Infinity):
                return _tmp485
            #RETURN_NEW(HEAD=NUMBER; DATA=_tmp485)
            #NEWINSTANCE(OBJ=_tmp492; HEAD=NUMBER; DATA=_tmp485)
            _tmp492 = cls(NUMBER, _tmp485)
            return _tmp492
        elif rhead is TERMS:
            #DIV_NUMBER_TERMS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_VALUE_TERMS(VALUE=ldata; RHS=other; RHSDATA=rdata)
            _tmp513 = ldata
            if not _tmp513:
                return cls.zero
            pairs = rdata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                c = div(ldata, c, cls)
                t = 1/t
                if c==1:
                    return t
                if t==cls.one:
                    return cls(NUMBER, c)
                #RETURN_NEW(HEAD=TERMS; DATA={t: c})
                #NEWINSTANCE(OBJ=_tmp520; HEAD=TERMS; DATA={t: c})
                _tmp520 = cls(TERMS, {t: c})
                return _tmp520
            #NEWINSTANCE(OBJ=_tmp513; HEAD=FACTORS; DATA={other: -1})
            _tmp513 = cls(FACTORS, {other: -1})
            if ldata==1:
                return _tmp513
            #RETURN_NEW(HEAD=TERMS; DATA={_tmp513: ldata})
            #NEWINSTANCE(OBJ=_tmp541; HEAD=TERMS; DATA={_tmp513: ldata})
            _tmp541 = cls(TERMS, {_tmp513: ldata})
            return _tmp541
        elif rhead is FACTORS:
            #DIV_NUMBER_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_VALUE_FACTORS(VALUE=ldata; RHS=other; RHSDATA=rdata)
            
            pairs = rdata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                c = -c
                if c==1:
                    return t * ldata
                new_pairs = {t: c}
            else:
                #NEG_DICT_VALUES(DICT_IN=pairs; DICT_OUT=new_pairs)
                new_pairs = dict([(t, -c) for t,c in pairs.iteritems()])
            #NEWINSTANCE(OBJ=_tmp562; HEAD=FACTORS; DATA=new_pairs)
            _tmp562 = cls(FACTORS, new_pairs)
            if ldata==1:
                return _tmp562
            #RETURN_NEW(HEAD=TERMS; DATA={_tmp562: ldata})
            #NEWINSTANCE(OBJ=_tmp583; HEAD=TERMS; DATA={_tmp562: ldata})
            _tmp583 = cls(TERMS, {_tmp562: ldata})
            return _tmp583
        else:
            #DIV_NUMBER_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_VALUE_SYMBOL(VALUE=ldata; RHS=other; RHSDATA=rdata)
            _tmp604 = ldata
            if not _tmp604:
                return cls.zero
            #NEWINSTANCE(OBJ=obj2; HEAD=FACTORS; DATA={other: -1})
            obj2 = cls(FACTORS, {other: -1})
            if _tmp604==1:
                return obj2
            #RETURN_NEW(HEAD=TERMS; DATA={obj2: _tmp604})
            #NEWINSTANCE(OBJ=_tmp618; HEAD=TERMS; DATA={obj2: _tmp604})
            _tmp618 = cls(TERMS, {obj2: _tmp604})
            return _tmp618
    elif lhead is TERMS:
        if rhead is NUMBER:
            #DIV_TERMS_NUMBER(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_TERMS_VALUE(VALUE=rdata; LHS=self; LHSDATA=ldata)
            #MUL_TERMS_VALUE(LHS=self; LHSDATA=ldata; VALUE=div(1,rdata,cls))
            #MUL_VALUE_TERMS(VALUE=div(1,rdata,cls); RHS=self; RHSDATA=ldata)
            _tmp653 = div(1,rdata,cls)
            if not _tmp653:
                return cls.zero
            pairs = ldata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                c = _tmp653 * c
                if c==1:
                    return t
                #RETURN_NEW(HEAD=TERMS; DATA={t: c})
                #NEWINSTANCE(OBJ=_tmp660; HEAD=TERMS; DATA={t: c})
                _tmp660 = cls(TERMS, {t: c})
                return _tmp660
            if _tmp653==1:
                return self
            pairs = {}
            for t,c in ldata.iteritems():
                pairs[t] = _tmp653 * c
            #NEWINSTANCE(OBJ=obj; HEAD=TERMS; DATA=pairs)
            obj = cls(TERMS, pairs)
            coeff, terms = self._coeff_terms
            if terms is not None:
                c = coeff * _tmp653
                if not c==1:
                    obj._coeff_terms = (c, terms)
            else:
                obj._coeff_terms = (_tmp653, self)
            return obj
        elif rhead is TERMS:
            #DIV_TERMS_TERMS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            
            if self==other:
                return cls.one
            lpairs = ldata
            rpairs = rdata
            if len(lpairs)==1:
                t1, c1 = lpairs.items()[0]
                if len(rpairs)==1:
                    t2, c2 = rpairs.items()[0]
                    c = div(c1, c2, cls)
                    if t2==t1:
                        return cls(NUMBER, c)
                    if c==1:
                        return t1 / t2
                        #@RETURN_NEW(HEAD=FACTORS; DATA={t1:1, t2:-1})
                    return (t1 / t2) * c
                    #@NEWINSTANCE(OBJ=_tmp681; HEAD=FACTORS; DATA={t1:1, t2:-1})
                else:
                    _tmp681 = t1 / other
                #RETURN_NEW(HEAD=TERMS; DATA={_tmp681:c1})
                #NEWINSTANCE(OBJ=_tmp688; HEAD=TERMS; DATA={_tmp681:c1})
                _tmp688 = cls(TERMS, {_tmp681:c1})
                return _tmp688
            elif len(rpairs)==1:
                t2, c2 = rpairs.items()[0]
                c = div(1, c2, cls)
                if t2==self:
                    return cls(NUMBER, c)
                _tmp681 = self / t2
                #RETURN_NEW(HEAD=TERMS; DATA={_tmp681:c})
                #NEWINSTANCE(OBJ=_tmp702; HEAD=TERMS; DATA={_tmp681:c})
                _tmp702 = cls(TERMS, {_tmp681:c})
                return _tmp702
            #RETURN_NEW(HEAD=FACTORS; DATA={self:1, other:-1})
            #NEWINSTANCE(OBJ=_tmp716; HEAD=FACTORS; DATA={self:1, other:-1})
            _tmp716 = cls(FACTORS, {self:1, other:-1})
            return _tmp716
        elif rhead is FACTORS:
            #DIV_TERMS_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            lpairs = ldata
            if len(lpairs)==1:
                t, c = lpairs.items()[0]
                t = t / other
                if t==cls.one:
                    return cls(NUMBER, c)
                head, data = t.pair
                if head is NUMBER:
                    #RETURN_NEW(HEAD=NUMBER; DATA=data * c)
                    #NEWINSTANCE(OBJ=_tmp737; HEAD=NUMBER; DATA=data * c)
                    _tmp737 = cls(NUMBER, data * c)
                    return _tmp737
                elif head is TERMS:
                    #MUL_TERMS_VALUE(LHS=t; LHSDATA=data; VALUE=c)
                    #MUL_VALUE_TERMS(VALUE=c; RHS=t; RHSDATA=data)
                    _tmp758 = c
                    if not _tmp758:
                        return cls.zero
                    pairs = data
                    if len(pairs)==1:
                        t, c = pairs.items()[0]
                        c = _tmp758 * c
                        if c==1:
                            return t
                        #RETURN_NEW(HEAD=TERMS; DATA={t: c})
                        #NEWINSTANCE(OBJ=_tmp765; HEAD=TERMS; DATA={t: c})
                        _tmp765 = cls(TERMS, {t: c})
                        return _tmp765
                    if _tmp758==1:
                        return t
                    pairs = {}
                    for t,c in data.iteritems():
                        pairs[t] = _tmp758 * c
                    #NEWINSTANCE(OBJ=obj; HEAD=TERMS; DATA=pairs)
                    obj = cls(TERMS, pairs)
                    coeff, terms = t._coeff_terms
                    if terms is not None:
                        c = coeff * _tmp758
                        if not c==1:
                            obj._coeff_terms = (c, terms)
                    else:
                        obj._coeff_terms = (_tmp758, t)
                    return obj
                else:
                    #MUL_SYMBOL_VALUE(LHS=t; LHSDATA=data; VALUE=c)
                    #MUL_VALUE_SYMBOL(VALUE=c; RHS=t; RHSDATA=data)
                    _tmp793 = c
                    if not _tmp793:
                        return cls.zero
                    if _tmp793==1:
                        return t
                    #RETURN_NEW(HEAD=TERMS; DATA={t: _tmp793})
                    #NEWINSTANCE(OBJ=_tmp800; HEAD=TERMS; DATA={t: _tmp793})
                    _tmp800 = cls(TERMS, {t: _tmp793})
                    return _tmp800
            #DIV_SYMBOL_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = rdata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                if t==self:
                    c = 1 - c
                    if not c:
                        return cls.one
                    if c==1:
                        return t
                    else:
                        #RETURN_NEW(HEAD=FACTORS; DATA={t: c})
                        #NEWINSTANCE(OBJ=_tmp821; HEAD=FACTORS; DATA={t: c})
                        _tmp821 = cls(FACTORS, {t: c})
                        return _tmp821
                #RETURN_NEW(HEAD=FACTORS; DATA={t: -c, self: 1})
                #NEWINSTANCE(OBJ=_tmp835; HEAD=FACTORS; DATA={t: -c, self: 1})
                _tmp835 = cls(FACTORS, {t: -c, self: 1})
                return _tmp835
            #NEG_DICT_VALUES(DICT_IN=rdata; DICT_OUT=pairs)
            pairs = dict([(t, -c) for t,c in rdata.iteritems()])
            #MUL_DICT_SYMBOL(DICT=pairs; RHS=self; RHSDATA=ldata)
            #ADD_TERM_VALUE_DICT(TERM=self; VALUE=1; DICT=pairs; DICT_GET=pairs.get; SIGN=+; USIGN=)
            _tmp863 = pairs.get(self)
            if _tmp863 is None:
                pairs[self] =  1
            else:
                _tmp863 = _tmp863 + 1
                if _tmp863:
                    pairs[self] = _tmp863
                else:
                    del pairs[self]
            #CANONIZE_FACTORS_DICT1(DICT=pairs)
            if not pairs:
                return cls.one
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   return t
               if t==cls.one:
                   return t
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp877; HEAD=FACTORS; DATA=pairs)
            _tmp877 = cls(FACTORS, pairs)
            return _tmp877
        else:
            #DIV_TERMS_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            
            pairs = ldata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                if t==other:
                    return cls(NUMBER, c)
                if t.head is FACTORS:
                    _tmp891 = t / other
                    #RETURN_NEW(HEAD=TERMS; DATA={_tmp891: c})
                    #NEWINSTANCE(OBJ=_tmp898; HEAD=TERMS; DATA={_tmp891: c})
                    _tmp898 = cls(TERMS, {_tmp891: c})
                    return _tmp898
                #NEWINSTANCE(OBJ=_tmp891; HEAD=FACTORS; DATA={t:1, other: -1})
                _tmp891 = cls(FACTORS, {t:1, other: -1})
                #RETURN_NEW(HEAD=TERMS; DATA={_tmp891: c})
                #NEWINSTANCE(OBJ=_tmp919; HEAD=TERMS; DATA={_tmp891: c})
                _tmp919 = cls(TERMS, {_tmp891: c})
                return _tmp919
            #RETURN_NEW(HEAD=FACTORS; DATA={self: 1, other: -1})
            #NEWINSTANCE(OBJ=_tmp933; HEAD=FACTORS; DATA={self: 1, other: -1})
            _tmp933 = cls(FACTORS, {self: 1, other: -1})
            return _tmp933
    elif lhead is FACTORS:
        if rhead is NUMBER:
            #DIV_FACTORS_NUMBER(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_FACTORS_VALUE(VALUE=rdata; LHS=self; LHSDATA=ldata)
            #MUL_FACTORS_VALUE(LHS=self; LHSDATA=ldata; VALUE=div(1,rdata,cls))
            #MUL_SYMBOL_VALUE(VALUE=div(1,rdata,cls); LHS=self; LHSDATA=ldata)
            #MUL_VALUE_SYMBOL(VALUE=div(1,rdata,cls); RHS=self; RHSDATA=ldata)
            _tmp975 = div(1,rdata,cls)
            if not _tmp975:
                return cls.zero
            if _tmp975==1:
                return self
            #RETURN_NEW(HEAD=TERMS; DATA={self: _tmp975})
            #NEWINSTANCE(OBJ=_tmp982; HEAD=TERMS; DATA={self: _tmp975})
            _tmp982 = cls(TERMS, {self: _tmp975})
            return _tmp982
        elif rhead is TERMS:
            #DIV_FACTORS_TERMS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            rpairs = rdata
            if len(rpairs)==1:
                t, c = rpairs.items()[0]
                t = self / t
                c = div(1, c, cls)
                if t==cls.one:
                    return cls(NUMBER, c)
                head, data = t.pair
                if head is NUMBER:
                    #RETURN_NEW(HEAD=NUMBER; DATA=data * c)
                    #NEWINSTANCE(OBJ=_tmp1003; HEAD=NUMBER; DATA=data * c)
                    _tmp1003 = cls(NUMBER, data * c)
                    return _tmp1003
                elif head is TERMS:
                    #MUL_TERMS_VALUE(LHS=t; LHSDATA=data; VALUE=c)
                    #MUL_VALUE_TERMS(VALUE=c; RHS=t; RHSDATA=data)
                    _tmp1024 = c
                    if not _tmp1024:
                        return cls.zero
                    pairs = data
                    if len(pairs)==1:
                        t, c = pairs.items()[0]
                        c = _tmp1024 * c
                        if c==1:
                            return t
                        #RETURN_NEW(HEAD=TERMS; DATA={t: c})
                        #NEWINSTANCE(OBJ=_tmp1031; HEAD=TERMS; DATA={t: c})
                        _tmp1031 = cls(TERMS, {t: c})
                        return _tmp1031
                    if _tmp1024==1:
                        return t
                    pairs = {}
                    for t,c in data.iteritems():
                        pairs[t] = _tmp1024 * c
                    #NEWINSTANCE(OBJ=obj; HEAD=TERMS; DATA=pairs)
                    obj = cls(TERMS, pairs)
                    coeff, terms = t._coeff_terms
                    if terms is not None:
                        c = coeff * _tmp1024
                        if not c==1:
                            obj._coeff_terms = (c, terms)
                    else:
                        obj._coeff_terms = (_tmp1024, t)
                    return obj
                else:
                    #MUL_SYMBOL_VALUE(LHS=t; LHSDATA=data; VALUE=c)
                    #MUL_VALUE_SYMBOL(VALUE=c; RHS=t; RHSDATA=data)
                    _tmp1059 = c
                    if not _tmp1059:
                        return cls.zero
                    if _tmp1059==1:
                        return t
                    #RETURN_NEW(HEAD=TERMS; DATA={t: _tmp1059})
                    #NEWINSTANCE(OBJ=_tmp1066; HEAD=TERMS; DATA={t: _tmp1059})
                    _tmp1066 = cls(TERMS, {t: _tmp1059})
                    return _tmp1066
            #DIV_FACTORS_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = ldata.copy()
            #DIV_DICT_SYMBOL(RHS=other; RHSDATA=rdata; DICT=pairs)
            #ADD_TERM_VALUE_DICT(TERM=other; VALUE=-1; DICT=pairs; DICT_GET=pairs.get; SIGN=+; USIGN=)
            _tmp1094 = pairs.get(other)
            if _tmp1094 is None:
                pairs[other] =  -1
            else:
                _tmp1094 = _tmp1094 + -1
                if _tmp1094:
                    pairs[other] = _tmp1094
                else:
                    del pairs[other]
            #CANONIZE_FACTORS_DICT1(DICT=pairs)
            if not pairs:
                return cls.one
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   return t
               if t==cls.one:
                   return t
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp1108; HEAD=FACTORS; DATA=pairs)
            _tmp1108 = cls(FACTORS, pairs)
            return _tmp1108
        elif rhead is FACTORS:
            #DIV_FACTORS_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = ldata.copy()
            pairs_get = pairs.get
            number = 1
            for t,c in rdata.iteritems():
                #MUL_FACTOR_VALUE_DICT(FACTOR=t; SIGN=-; USIGN=-; VALUE=c; DICT=pairs; DICT_GET=pairs_get; NUMBER=number)
                _tmp1129 = pairs_get(t)
                if _tmp1129 is None:
                    pairs[t] = - c
                else:
                    _tmp1129 = _tmp1129 - c
                    if type(_tmp1129) is cls and _tmp1129.head is NUMBER:
                        _tmp1129 = _tmp1129.data
                    if _tmp1129:
                        if t.head is NUMBER:
                            del pairs[t]
                            z, sym = try_power(t.data, _tmp1129)
                            if sym:
                                for t1, c1 in sym:
                                    #NEWINSTANCE(OBJ=tt; HEAD=NUMBER; DATA=t1)
                                    tt = cls(NUMBER, t1)
                                    #ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=tt; VALUE=c1; SIGN=+; USIGN=)
                                    _tmp1143 = pairs_get(tt)
                                    if _tmp1143 is None:
                                        pairs[tt] =  c1
                                    else:
                                        _tmp1143 = _tmp1143 + c1
                                        if _tmp1143:
                                            pairs[tt] = _tmp1143
                                        else:
                                            del pairs[tt]
                            number = number * z
                        else:
                            pairs[t] = _tmp1129
                    else:
                        del pairs[t]
            #CANONIZE_FACTORS_DICT(DICT=pairs; NUMBER=number)
            if not pairs:
                if number is 1:
                    return cls.one
                return number
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   if number==1:
                       return t
                   #RETURN_NEW(HEAD=TERMS; DATA={t: number})
                   #NEWINSTANCE(OBJ=_tmp1157; HEAD=TERMS; DATA={t: number})
                   _tmp1157 = cls(TERMS, {t: number})
                   return _tmp1157
               if t==cls.one:
                   return number
            if number==1:
                #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
                #NEWINSTANCE(OBJ=_tmp1171; HEAD=FACTORS; DATA=pairs)
                _tmp1171 = cls(FACTORS, pairs)
                return _tmp1171
            #NEWINSTANCE(OBJ=_tmp1122; HEAD=FACTORS; DATA=pairs)
            _tmp1122 = cls(FACTORS, pairs)
            #RETURN_NEW(HEAD=TERMS; DATA={_tmp1122: number})
            #NEWINSTANCE(OBJ=_tmp1192; HEAD=TERMS; DATA={_tmp1122: number})
            _tmp1192 = cls(TERMS, {_tmp1122: number})
            return _tmp1192
        else:
            #DIV_FACTORS_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = ldata.copy()
            #DIV_DICT_SYMBOL(RHS=other; RHSDATA=rdata; DICT=pairs)
            #ADD_TERM_VALUE_DICT(TERM=other; VALUE=-1; DICT=pairs; DICT_GET=pairs.get; SIGN=+; USIGN=)
            _tmp1220 = pairs.get(other)
            if _tmp1220 is None:
                pairs[other] =  -1
            else:
                _tmp1220 = _tmp1220 + -1
                if _tmp1220:
                    pairs[other] = _tmp1220
                else:
                    del pairs[other]
            #CANONIZE_FACTORS_DICT1(DICT=pairs)
            if not pairs:
                return cls.one
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   return t
               if t==cls.one:
                   return t
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp1234; HEAD=FACTORS; DATA=pairs)
            _tmp1234 = cls(FACTORS, pairs)
            return _tmp1234
    else:
        if rhead is NUMBER:
            #DIV_SYMBOL_NUMBER(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            #DIV_SYMBOL_VALUE(VALUE=rdata; LHS=self; LHSDATA=ldata)
            #MUL_VALUE_SYMBOL(VALUE=div(1, rdata, cls); RHS=self; RHSDATA=ldata)
            _tmp1262 = div(1, rdata, cls)
            if not _tmp1262:
                return cls.zero
            if _tmp1262==1:
                return self
            #RETURN_NEW(HEAD=TERMS; DATA={self: _tmp1262})
            #NEWINSTANCE(OBJ=_tmp1269; HEAD=TERMS; DATA={self: _tmp1262})
            _tmp1269 = cls(TERMS, {self: _tmp1262})
            return _tmp1269
        elif rhead is TERMS:
            #DIV_SYMBOL_TERMS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = rdata
            if len(pairs)==1:
                t,c = pairs.items()[0]
                if self==t:
                    return cls(NUMBER, div(1, c, cls))
                #NEWINSTANCE(OBJ=_tmp1283; HEAD=FACTORS; DATA={self:1, t:-1})
                _tmp1283 = cls(FACTORS, {self:1, t:-1})
                #RETURN_NEW(HEAD=TERMS; DATA={_tmp1283: div(1, c, cls)})
                #NEWINSTANCE(OBJ=_tmp1297; HEAD=TERMS; DATA={_tmp1283: div(1, c, cls)})
                _tmp1297 = cls(TERMS, {_tmp1283: div(1, c, cls)})
                return _tmp1297
            #RETURN_NEW(HEAD=FACTORS; DATA={self:1, other:-1})
            #NEWINSTANCE(OBJ=_tmp1311; HEAD=FACTORS; DATA={self:1, other:-1})
            _tmp1311 = cls(FACTORS, {self:1, other:-1})
            return _tmp1311
        elif rhead is FACTORS:
            #DIV_SYMBOL_FACTORS(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            pairs = rdata
            if len(pairs)==1:
                t, c = pairs.items()[0]
                if t==self:
                    c = 1 - c
                    if not c:
                        return cls.one
                    if c==1:
                        return t
                    else:
                        #RETURN_NEW(HEAD=FACTORS; DATA={t: c})
                        #NEWINSTANCE(OBJ=_tmp1332; HEAD=FACTORS; DATA={t: c})
                        _tmp1332 = cls(FACTORS, {t: c})
                        return _tmp1332
                #RETURN_NEW(HEAD=FACTORS; DATA={t: -c, self: 1})
                #NEWINSTANCE(OBJ=_tmp1346; HEAD=FACTORS; DATA={t: -c, self: 1})
                _tmp1346 = cls(FACTORS, {t: -c, self: 1})
                return _tmp1346
            #NEG_DICT_VALUES(DICT_IN=rdata; DICT_OUT=pairs)
            pairs = dict([(t, -c) for t,c in rdata.iteritems()])
            #MUL_DICT_SYMBOL(DICT=pairs; RHS=self; RHSDATA=ldata)
            #ADD_TERM_VALUE_DICT(TERM=self; VALUE=1; DICT=pairs; DICT_GET=pairs.get; SIGN=+; USIGN=)
            _tmp1374 = pairs.get(self)
            if _tmp1374 is None:
                pairs[self] =  1
            else:
                _tmp1374 = _tmp1374 + 1
                if _tmp1374:
                    pairs[self] = _tmp1374
                else:
                    del pairs[self]
            #CANONIZE_FACTORS_DICT1(DICT=pairs)
            if not pairs:
                return cls.one
            if len(pairs)==1:
               t, c = pairs.items()[0]
               if c==1:
                   return t
               if t==cls.one:
                   return t
            #RETURN_NEW(HEAD=FACTORS; DATA=pairs)
            #NEWINSTANCE(OBJ=_tmp1388; HEAD=FACTORS; DATA=pairs)
            _tmp1388 = cls(FACTORS, pairs)
            return _tmp1388
        else:
            #DIV_SYMBOL_SYMBOL(LHS=self; LHSDATA=ldata; RHS=other; RHSDATA=rdata)
            if self == other:
                return cls.one
            #RETURN_NEW(HEAD=FACTORS; DATA={self: 1, other: -1})
            #NEWINSTANCE(OBJ=_tmp1409; HEAD=FACTORS; DATA={self: 1, other: -1})
            _tmp1409 = cls(FACTORS, {self: 1, other: -1})
            return _tmp1409



""" Provides CalculusInfinity class.
"""
__docformat__ = "restructuredtext"
__all__ = ['CalculusInfinity', 'oo', 'moo', 'undefined', 'zoo']

from ..utils import str_PRODUCT, NUMBER, TERMS, FACTORS, SYMBOL
from ..heads import SPECIAL
from ..arithmetic import Infinity
from .algebra import Calculus

CalculusInfinity = Infinity

class _CalculusInfinity(Infinity):

    one = Calculus.one
    zero = Calculus.zero

    def __new__(cls, direction):
        if isinstance(direction, Calculus):
            head, data = direction.head, direction.data
            if head is NUMBER:
                return cls(data)
            elif head is TERMS:
                if len(data)==1:
                    t, c = data.items()[0]
                    if c > 0:
                        return cls(t)
            r = direction.get_direction()
            if r is not NotImplemented:
                return cls(r)
        return Infinity.__new__(cls, direction)

    def to_str_data(self, sort=True):
        return str_PRODUCT, str(self)

    def _get_symbols_data(self):
        return set()

    def subs(self, *args):
        d = self.data
        if isinstance(d, Calculus):
            return type(self)(d.subs(*args))
        return self

    @classmethod
    def IsUnbounded(cls, x):
        if isinstance(x, Calculus):
            head = x.head
            if head is NUMBER:
                return cls.IsUnbounded(x.data)
            f = x.is_bounded
            if f or f is not None:
                return Calculus.convert(not f)
        if isinstance(x, cls):
            return Calculus.one
        r = Infinity.IsUnbounded(x)
        if r is not NotImplemented:
            return Calculus.convert(r)
        x = Calculus.convert(x)
        return Calculus.Apply(cls.IsUnbounded, x)

    @classmethod
    def EqualArg(cls, x, y):
        if isinstance(x, Calculus):
            d = x.get_direction()
            if d is not NotImplemented:
                return cls.EqualArg(d, y)
        if isinstance(y, Calculus):
            d = y.get_direction()
            if d is not NotImplemented:
                return cls.EqualArg(x, d)
            if isinstance(x, Calculus):
                xy = x / y
                if xy.head is NUMBER:
                    return Calculus.convert(xy.data > 0)

        r = Infinity.EqualArg(x, y)
        if r is not NotImplemented:
            return Calculus.convert(r)
        x = Calculus.convert(x)
        y = Calculus.convert(y)
        return Calculus.Apply(cls.EqualArg, x,y)

    @classmethod
    def IsPositive(cls, x):
        if isinstance(x, Calculus):
            if x.head is NUMBER:
                return cls.IsPositive(x.data)
        r = Infinity.IsPositive(x)
        if r is not NotImplemented:
            return Calculus.convert(r)
        x = Calculus.convert(x)
        return Calculus.apply(cls.IsPositive, x)

    def __pow__(self, other):
        if isinstance(other, Calculus):
            head, data = other.head, other.data
            if head is NUMBER:
                other = data
        r = Infinity.__pow__(self, other)
        if r is not NotImplemented:
            if isinstance(r, type(self)):
                return r
            return Calculus.convert(r)
        x = Calculus.convert(other)
        return Calculus(FACTORS, {Calculus(SPECIAL, self): x})

    def __rpow__(self, other):
        if isinstance(other, Calculus):
            if other.head is SYMBOL:
                other = other.evalf(2)
            head, data = other.head, other.data
            if head is NUMBER:
                other = data
        r = Infinity.__rpow__(self, other)
        if r is not NotImplemented:
            if isinstance(r, type(self)):
                return r
            return Calculus.convert(r)
        x = Calculus.convert(other)
        return Calculus(FACTORS, {x: self})

oo = CalculusInfinity(1)
moo = CalculusInfinity(-1)
undefined = CalculusInfinity(0)
zoo = CalculusInfinity(undefined)

Calculus.oo = oo
Calculus.zoo = zoo
Calculus.undefined = undefined
Calculus.Infinity = CalculusInfinity



'''
.. module:: steadystate
   :platform: Linux
   :synopsis: The steadystate module currently contains a shell class for sub-branching as well as two classes for calculating the
              steady state of DSGE models. One of them computes steady states using closed form information, while the other one makes
              use of a nonlinear root-finding algorithm supplied by the scipy optimize package.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''
import re
import numpy as np
import copy
from scipy import optimize
# Switch off runtime warnings here
import warnings
warnings.filterwarnings("ignore")

class SSsolvers(object):
    def __init__(self):
        pass

class ManualSteadyState(SSsolvers):
    """
    Solve for the steady state manually
    """
    def __init__(self,intup,other=None):
        self.manss_sys = intup[0]
        self.paramdic = intup[1]
        self.other = other

    def solve(self):
        list_tmp1 = copy.deepcopy(self.manss_sys)
        # Create manual (closed-form) steady state dictionary
        _rlog = 'LOG\('
        _rexp = 'EXP\('
        rlog = re.compile(_rlog)
        rexp = re.compile(_rexp)
        manss={}
        locals().update(self.paramdic)
        globals().update(self.paramdic)
        for x in list_tmp1:
            str_tmp3 = x[:]
            while rlog.search(str_tmp3):
                str_tmp3 = re.sub(rlog,'np.log(',str_tmp3)
            while rexp.search(str_tmp3):
                str_tmp3 = re.sub(rexp,'np.exp(',str_tmp3)           
            str_tmp = str_tmp3.split(';')[0]
            list_tmp = str_tmp.split('=')
            str_tmp1 = list_tmp[0].strip()
            str_tmp2 = list_tmp[1].strip()
            manss[str_tmp1] = eval(str_tmp2)
            locals()[str_tmp1] = eval(str_tmp2)
            globals().update(manss)
        self.sstate = manss

class Fsolve(SSsolvers):
    def __init__(self,intup,other=None):
        self.ssm = intup[0]
        self.ssi = intup[1]
        self.paramdic = intup[2]
        self.other = other
        # create a dic holding non _bar values in ssi
        tmp_dic = {}
        for keyo in self.ssi.keys():
            if '_bar' not in keyo: tmp_dic[keyo] = self.ssi[keyo]
        self.nonbardic = tmp_dic

    def solve(self,bounds_dic=None):

        # Turn the non-linear system into a representation
        # that is suitable for fsolve(invar) !
        patup = ('{-10,10}|None','iid','{-10,10}')
        ssi = self.ssi
        subdic = {}
        for y,z in zip(ssi.items(),range(len(ssi.items()))):
            subdic[y[0]] = (y[0],y[1],z)
            
        list_tmp1 = copy.deepcopy(self.ssm)
        for var in ssi.keys():
            _mreg = '(\+|\*|-|/|^|\()'+var
            mreg = re.compile(_mreg)
            _rlog = 'LOG\('
            _rexp = 'EXP\('
            rlog = re.compile(_rlog)
            rexp = re.compile(_rexp)
            for i1,line in enumerate(list_tmp1):
                # First strip away any whitespace
                while ' ' in list_tmp1[i1]:
                    list_tmp1[i1] = list_tmp1[i1].replace(' ','')
                while rlog.search(list_tmp1[i1]):
                    list_tmp1[i1] = re.sub(rlog,'np.log(',list_tmp1[i1])
                while rexp.search(list_tmp1[i1]):
                    list_tmp1[i1] = re.sub(rexp,'np.exp(',list_tmp1[i1])
                # Eliminate IID variables by setting equal to zero
                while self.other.vreg(patup,list_tmp1[i1],True,'max'):
                    varli = self.other.vreg(patup,list_tmp1[i1],True,'max')
                    varli.reverse()
                    for varo in varli:
                        pos = varo[3][0]
                        poe = varo[3][1]
                        vari = varo[0]
                        list_tmp1[i1] = list_tmp1[i1][:pos]+'0.0'+list_tmp1[i1][poe:]                        
                while mreg.search(list_tmp1[i1]):
                    ma = mreg.search(list_tmp1[i1])
                    matot = ma.group()
                    if len(ma.group(1))>0:
                        var = ma.group()[1:]
                        pos = ma.span()[0]+1
                    else:
                        var = ma.group()
                        pos = ma.span()[0]                      
                    poe = ma.span()[1]
                    list_tmp1[i1] = list_tmp1[i1][:pos]+'invar['+str(subdic[var][2])+']'+list_tmp1[i1][poe:]
        func_repr = list_tmp1
        self.subdic = copy.deepcopy(subdic)
        self.ssm_alt = copy.deepcopy(func_repr)

        # Define the function to be handed over
        # to fsolve
        if bounds_dic == None:
            def func(invar):
                locals().update(self.paramdic)
                locals().update(self.nonbardic)
                fdot = np.zeros((len(func_repr)),float)
                for i1,x in enumerate(func_repr):
                    fdot[i1] = eval(x)
                return fdot
        else:
            def func(invar):
                locals().update(self.paramdic)
                locals().update(self.nonbardic)
                fdot = np.zeros((len(func_repr)),float)
                for i1,x in enumerate(func_repr):
                    fdot[i1] = eval(x)
                return np.sum(np.abs(fdot))
            

        # Define the initial values and
        # start the non-linear solver
        inlist = []
        for x in subdic.values():
            inlist.append([x[2],x[1],x[0]])
        inlist.sort()
        init_val = [float(x[1]) for x in inlist]

        # Prepare the case of constrained optimisation if needed
        blist = []
        if bounds_dic != None:
            for elem in inlist:
                if elem[2] in bounds_dic.keys(): blist.append(bounds_dic[elem[2]])
                else: blist.append((None,None))

        # Accomodate unconstrained and constrained optimisation
        if bounds_dic == None:
            outobj = optimize.root(func,init_val,method='hybr')
            output = outobj.x
            mesg = outobj.message
            ier = outobj.status
        else:
            outobj = optimize.minimize(func,init_val,method='L-BFGS-B',bounds=blist)
            output = outobj.x
            mesg = outobj.message
            ier = outobj.status
        # Attach the outputs of the solver as attributes
        self.fsout={}
        for x,y in zip(output,inlist):
            self.fsout[y[2]] = x
        self.ier = ier
        self.mesg = mesg
        self.output = output
